@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,32 +9,30 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
 #include "stylehelper.h"
 
+#include "theme/theme.h"
 #include "hostosinfo.h"
 
 #include <QPixmapCache>
 #include <QPainter>
 #include <QApplication>
+#include <QFileInfo>
+#include <QCommonStyle>
 #include <QStyleOption>
 #include <qmath.h>
 
@@ -150,10 +148,20 @@ void StyleHelper::setBaseColor(const QColor &newcolor)
 {
     m_requestedBaseColor = newcolor;
 
+    const QColor themeBaseColor = creatorTheme()->color(Theme::PanelStatusBarBackgroundColor);
+    const QColor defaultBaseColor = QColor(DEFAULT_BASE_COLOR);
     QColor color;
-    color.setHsv(newcolor.hue(),
-                 newcolor.saturation() * 0.7,
-                 64 + newcolor.value() / 3);
+
+    if (defaultBaseColor == newcolor) {
+        color = themeBaseColor;
+    } else {
+        const int valueDelta = (newcolor.value() - defaultBaseColor.value()) / 3;
+        const int value = qBound(0, themeBaseColor.value() + valueDelta, 255);
+
+        color.setHsv(newcolor.hue(),
+                     newcolor.saturation() * 0.7,
+                     value);
+    }
 
     if (color.isValid() && color != m_baseColor) {
         m_baseColor = color;
@@ -274,74 +282,45 @@ static void menuGradientHelper(QPainter *p, const QRect &spanRect, const QRect &
 
 void StyleHelper::drawArrow(QStyle::PrimitiveElement element, QPainter *painter, const QStyleOption *option)
 {
-    // From windowsstyle but modified to enable AA
     if (option->rect.width() <= 1 || option->rect.height() <= 1)
         return;
 
+    const qreal devicePixelRatio = painter->device()->devicePixelRatio();
+    const bool enabled = option->state & QStyle::State_Enabled;
     QRect r = option->rect;
     int size = qMin(r.height(), r.width());
     QPixmap pixmap;
     QString pixmapName;
-    pixmapName.sprintf("arrow-%s-%d-%d-%d-%lld",
-                       "$qt_ia",
-                       uint(option->state), element,
-                       size, option->palette.cacheKey());
+    pixmapName.sprintf("StyleHelper::drawArrow-%d-%d-%d-%f",
+                       element, size, enabled, devicePixelRatio);
     if (!QPixmapCache::find(pixmapName, pixmap)) {
-        int border = size/5;
-        int sqsize = 2*(size/2);
-        QImage image(sqsize, sqsize, QImage::Format_ARGB32);
+        QImage image(size * devicePixelRatio, size * devicePixelRatio, QImage::Format_ARGB32_Premultiplied);
         image.fill(Qt::transparent);
-        QPainter imagePainter(&image);
-        imagePainter.setRenderHint(QPainter::Antialiasing, true);
-        imagePainter.translate(0.5, 0.5);
-        QPolygon a;
-        switch (element) {
-            case QStyle::PE_IndicatorArrowUp:
-                a.setPoints(3, border, sqsize/2,  sqsize/2, border,  sqsize - border, sqsize/2);
-                break;
-            case QStyle::PE_IndicatorArrowDown:
-                a.setPoints(3, border, sqsize/2,  sqsize/2, sqsize - border,  sqsize - border, sqsize/2);
-                break;
-            case QStyle::PE_IndicatorArrowRight:
-                a.setPoints(3, sqsize - border, sqsize/2,  sqsize/2, border,  sqsize/2, sqsize - border);
-                break;
-            case QStyle::PE_IndicatorArrowLeft:
-                a.setPoints(3, border, sqsize/2,  sqsize/2, border,  sqsize/2, sqsize - border);
-                break;
-            default:
-                break;
-        }
+        QPainter painter(&image);
 
-        int bsx = 0;
-        int bsy = 0;
+        QStyleOption tweakedOption(*option);
+        tweakedOption.state = QStyle::State_Enabled;
 
-        if (option->state & QStyle::State_Sunken) {
-            bsx = qApp->style()->pixelMetric(QStyle::PM_ButtonShiftHorizontal);
-            bsy = qApp->style()->pixelMetric(QStyle::PM_ButtonShiftVertical);
-        }
+        auto drawCommonStyleArrow = [&tweakedOption, element, &painter](const QRect &rect, const QColor &color) -> void
+        {
+            static const QCommonStyle* const style = qobject_cast<QCommonStyle*>(QApplication::style());
+            if (!style)
+                return;
+            tweakedOption.palette.setColor(QPalette::ButtonText, color.rgb());
+            tweakedOption.rect = rect;
+            painter.setOpacity(color.alphaF());
+            style->QCommonStyle::drawPrimitive(element, &tweakedOption, &painter);
+        };
 
-        QRect bounds = a.boundingRect();
-        int sx = sqsize / 2 - bounds.center().x() - 1;
-        int sy = sqsize / 2 - bounds.center().y() - 1;
-        imagePainter.translate(sx + bsx, sy + bsy);
-
-        if (!(option->state & QStyle::State_Enabled)) {
-            imagePainter.setBrush(option->palette.mid().color());
-            imagePainter.setPen(option->palette.mid().color());
+        if (!enabled) {
+            drawCommonStyleArrow(image.rect(), creatorTheme()->color(Theme::IconsDisabledColor));
         } else {
-            QColor shadow(0, 0, 0, 100);
-            imagePainter.translate(0, 1);
-            imagePainter.setPen(shadow);
-            imagePainter.setBrush(shadow);
-            QColor foreGround(255, 255, 255, 210);
-            imagePainter.drawPolygon(a);
-            imagePainter.translate(0, -1);
-            imagePainter.setPen(foreGround);
-            imagePainter.setBrush(foreGround);
+            drawCommonStyleArrow(image.rect().translated(0, devicePixelRatio), toolBarDropShadowColor());
+            drawCommonStyleArrow(image.rect(), creatorTheme()->color(Theme::IconsBaseColor));
         }
-        imagePainter.drawPolygon(a);
-        imagePainter.end();
+        painter.end();
         pixmap = QPixmap::fromImage(image);
+        pixmap.setDevicePixelRatio(devicePixelRatio);
         QPixmapCache::insert(pixmapName, pixmap);
     }
     int xOffset = r.x() + (r.width() - size)/2;
@@ -373,14 +352,19 @@ void StyleHelper::menuGradient(QPainter *painter, const QRect &spanRect, const Q
     }
 }
 
-static qreal pixmapDevicePixelRatio(const QPixmap &pixmap)
+QPixmap StyleHelper::disabledSideBarIcon(const QPixmap &enabledicon)
 {
-#if QT_VERSION > 0x050000
-    return pixmap.devicePixelRatio();
-#else
-    Q_UNUSED(pixmap);
-    return 1.0;
-#endif
+    QImage im = enabledicon.toImage().convertToFormat(QImage::Format_ARGB32);
+    for (int y=0; y<im.height(); ++y) {
+        QRgb *scanLine = reinterpret_cast<QRgb*>(im.scanLine(y));
+        for (int x=0; x<im.width(); ++x) {
+            QRgb pixel = *scanLine;
+            char intensity = char(qGray(pixel));
+            *scanLine = qRgba(intensity, intensity, intensity, qAlpha(pixel));
+            ++scanLine;
+        }
+    }
+    return QPixmap::fromImage(im);
 }
 
 // Draws a cached pixmap with shadow
@@ -396,8 +380,9 @@ void StyleHelper::drawIconWithShadow(const QIcon &icon, const QRect &rect,
         // return a high-dpi pixmap, which will in that case have a devicePixelRatio
         // different than 1. The shadow drawing caluculations are done in device
         // pixels.
-        QPixmap px = icon.pixmap(rect.size());
-        int devicePixelRatio = qCeil(pixmapDevicePixelRatio(px));
+        QWindow *window = QApplication::allWidgets().first()->windowHandle();
+        QPixmap px = icon.pixmap(window, rect.size(), iconMode);
+        int devicePixelRatio = qCeil(px.devicePixelRatio());
         int radius = dipRadius * devicePixelRatio;
         QPoint offset = dipOffset * devicePixelRatio;
         cache = QPixmap(px.size() + QSize(radius * 2, radius * 2));
@@ -405,103 +390,101 @@ void StyleHelper::drawIconWithShadow(const QIcon &icon, const QRect &rect,
 
         QPainter cachePainter(&cache);
         if (iconMode == QIcon::Disabled) {
-            QImage im = px.toImage().convertToFormat(QImage::Format_ARGB32);
-            for (int y=0; y<im.height(); ++y) {
-                QRgb *scanLine = (QRgb*)im.scanLine(y);
-                for (int x=0; x<im.width(); ++x) {
-                    QRgb pixel = *scanLine;
-                    char intensity = qGray(pixel);
-                    *scanLine = qRgba(intensity, intensity, intensity, qAlpha(pixel));
-                    ++scanLine;
-                }
-            }
-            px = QPixmap::fromImage(im);
+            const bool hasDisabledState = icon.availableSizes(QIcon::Disabled).contains(px.size());
+            if (!hasDisabledState)
+                px = disabledSideBarIcon(icon.pixmap(window, rect.size()));
+        } else {
+            // Draw shadow
+            QImage tmp(px.size() + QSize(radius * 2, radius * 2 + 1), QImage::Format_ARGB32_Premultiplied);
+            tmp.fill(Qt::transparent);
+
+            QPainter tmpPainter(&tmp);
+            tmpPainter.setCompositionMode(QPainter::CompositionMode_Source);
+            tmpPainter.drawPixmap(QRect(radius, radius, px.width(), px.height()), px);
+            tmpPainter.end();
+
+            // blur the alpha channel
+            QImage blurred(tmp.size(), QImage::Format_ARGB32_Premultiplied);
+            blurred.fill(Qt::transparent);
+            QPainter blurPainter(&blurred);
+            qt_blurImage(&blurPainter, tmp, radius, false, true);
+            blurPainter.end();
+
+            tmp = blurred;
+
+            // blacken the image...
+            tmpPainter.begin(&tmp);
+            tmpPainter.setCompositionMode(QPainter::CompositionMode_SourceIn);
+            tmpPainter.fillRect(tmp.rect(), color);
+            tmpPainter.end();
+
+            tmpPainter.begin(&tmp);
+            tmpPainter.setCompositionMode(QPainter::CompositionMode_SourceIn);
+            tmpPainter.fillRect(tmp.rect(), color);
+            tmpPainter.end();
+
+            // draw the blurred drop shadow...
+            cachePainter.drawImage(QRect(0, 0, cache.rect().width(), cache.rect().height()), tmp);
         }
 
-        // Draw shadow
-        QImage tmp(px.size() + QSize(radius * 2, radius * 2 + 1), QImage::Format_ARGB32_Premultiplied);
-        tmp.fill(Qt::transparent);
-
-        QPainter tmpPainter(&tmp);
-        tmpPainter.setCompositionMode(QPainter::CompositionMode_Source);
-        tmpPainter.drawPixmap(QRect(radius, radius, px.width(), px.height()), px);
-        tmpPainter.end();
-
-        // blur the alpha channel
-        QImage blurred(tmp.size(), QImage::Format_ARGB32_Premultiplied);
-        blurred.fill(Qt::transparent);
-        QPainter blurPainter(&blurred);
-        qt_blurImage(&blurPainter, tmp, radius, false, true);
-        blurPainter.end();
-
-        tmp = blurred;
-
-        // blacken the image...
-        tmpPainter.begin(&tmp);
-        tmpPainter.setCompositionMode(QPainter::CompositionMode_SourceIn);
-        tmpPainter.fillRect(tmp.rect(), color);
-        tmpPainter.end();
-
-        tmpPainter.begin(&tmp);
-        tmpPainter.setCompositionMode(QPainter::CompositionMode_SourceIn);
-        tmpPainter.fillRect(tmp.rect(), color);
-        tmpPainter.end();
-
-        // draw the blurred drop shadow...
-        cachePainter.drawImage(QRect(0, 0, cache.rect().width(), cache.rect().height()), tmp);
-
         // Draw the actual pixmap...
         cachePainter.drawPixmap(QRect(QPoint(radius, radius) + offset, QSize(px.width(), px.height())), px);
-#if QT_VERSION > 0x050000
         cache.setDevicePixelRatio(devicePixelRatio);
-#endif
         QPixmapCache::insert(pixmapName, cache);
     }
 
     QRect targetRect = cache.rect();
-    targetRect.setSize(targetRect.size() / pixmapDevicePixelRatio(cache));
+    targetRect.setSize(targetRect.size() / cache.devicePixelRatio());
     targetRect.moveCenter(rect.center() - dipOffset);
     p->drawPixmap(targetRect, cache);
 }
 
 // Draws a CSS-like border image where the defined borders are not stretched
+// Unit for rect, left, top, right and bottom is user pixels
 void StyleHelper::drawCornerImage(const QImage &img, QPainter *painter, const QRect &rect,
                                   int left, int top, int right, int bottom)
 {
-    QSize size = img.size();
+    // source rect for drawImage() calls needs to be specified in DIP unit of the image
+    const qreal imagePixelRatio = img.devicePixelRatio();
+    const qreal leftDIP = left * imagePixelRatio;
+    const qreal topDIP = top * imagePixelRatio;
+    const qreal rightDIP = right * imagePixelRatio;
+    const qreal bottomDIP = bottom * imagePixelRatio;
+
+    const QSize size = img.size();
     if (top > 0) { //top
-        painter->drawImage(QRect(rect.left() + left, rect.top(), rect.width() -right - left, top), img,
-                           QRect(left, 0, size.width() -right - left, top));
+        painter->drawImage(QRectF(rect.left() + left, rect.top(), rect.width() -right - left, top), img,
+                           QRectF(leftDIP, 0, size.width() - rightDIP - leftDIP, topDIP));
         if (left > 0) //top-left
-            painter->drawImage(QRect(rect.left(), rect.top(), left, top), img,
-                               QRect(0, 0, left, top));
+            painter->drawImage(QRectF(rect.left(), rect.top(), left, top), img,
+                               QRectF(0, 0, leftDIP, topDIP));
         if (right > 0) //top-right
-            painter->drawImage(QRect(rect.left() + rect.width() - right, rect.top(), right, top), img,
-                               QRect(size.width() - right, 0, right, top));
+            painter->drawImage(QRectF(rect.left() + rect.width() - right, rect.top(), right, top), img,
+                               QRectF(size.width() - rightDIP, 0, rightDIP, topDIP));
     }
     //left
     if (left > 0)
-        painter->drawImage(QRect(rect.left(), rect.top()+top, left, rect.height() - top - bottom), img,
-                           QRect(0, top, left, size.height() - bottom - top));
+        painter->drawImage(QRectF(rect.left(), rect.top()+top, left, rect.height() - top - bottom), img,
+                           QRectF(0, topDIP, leftDIP, size.height() - bottomDIP - topDIP));
     //center
-    painter->drawImage(QRect(rect.left() + left, rect.top()+top, rect.width() -right - left,
-                             rect.height() - bottom - top), img,
-                       QRect(left, top, size.width() -right -left,
-                             size.height() - bottom - top));
+    painter->drawImage(QRectF(rect.left() + left, rect.top()+top, rect.width() -right - left,
+                              rect.height() - bottom - top), img,
+                       QRectF(leftDIP, topDIP, size.width() - rightDIP - leftDIP,
+                              size.height() - bottomDIP - topDIP));
     if (right > 0) //right
-        painter->drawImage(QRect(rect.left() +rect.width() - right, rect.top()+top, right, rect.height() - top - bottom), img,
-                           QRect(size.width() - right, top, right, size.height() - bottom - top));
+        painter->drawImage(QRectF(rect.left() +rect.width() - right, rect.top()+top, right, rect.height() - top - bottom), img,
+                           QRectF(size.width() - rightDIP, topDIP, rightDIP, size.height() - bottomDIP - topDIP));
     if (bottom > 0) { //bottom
-        painter->drawImage(QRect(rect.left() +left, rect.top() + rect.height() - bottom,
-                                 rect.width() - right - left, bottom), img,
-                           QRect(left, size.height() - bottom,
-                                 size.width() - right - left, bottom));
-    if (left > 0) //bottom-left
-        painter->drawImage(QRect(rect.left(), rect.top() + rect.height() - bottom, left, bottom), img,
-                           QRect(0, size.height() - bottom, left, bottom));
-    if (right > 0) //bottom-right
-        painter->drawImage(QRect(rect.left() + rect.width() - right, rect.top() + rect.height() - bottom, right, bottom), img,
-                           QRect(size.width() - right, size.height() - bottom, right, bottom));
+        painter->drawImage(QRectF(rect.left() +left, rect.top() + rect.height() - bottom,
+                                  rect.width() - right - left, bottom), img,
+                           QRectF(leftDIP, size.height() - bottomDIP,
+                                  size.width() - rightDIP - leftDIP, bottomDIP));
+        if (left > 0) //bottom-left
+            painter->drawImage(QRectF(rect.left(), rect.top() + rect.height() - bottom, left, bottom), img,
+                               QRectF(0, size.height() - bottomDIP, leftDIP, bottomDIP));
+        if (right > 0) //bottom-right
+            painter->drawImage(QRectF(rect.left() + rect.width() - right, rect.top() + rect.height() - bottom, right, bottom), img,
+                               QRectF(size.width() - rightDIP, size.height() - bottomDIP, rightDIP, bottomDIP));
     }
 }
 
@@ -538,4 +521,35 @@ QLinearGradient StyleHelper::statusBarGradient(const QRect &statusBarRect)
     return grad;
 }
 
+QString StyleHelper::dpiSpecificImageFile(const QString &fileName)
+{
+    // See QIcon::addFile()
+    if (qApp->devicePixelRatio() > 1.0) {
+        const QString atDprfileName =
+                imageFileWithResolution(fileName, qRound(qApp->devicePixelRatio()));
+        if (QFile::exists(atDprfileName))
+            return atDprfileName;
+    }
+    return fileName;
+}
+
+QString StyleHelper::imageFileWithResolution(const QString &fileName, int dpr)
+{
+    const QFileInfo fi(fileName);
+    return dpr == 1 ? fileName :
+                      fi.path() + QLatin1Char('/') + fi.completeBaseName()
+                      + QLatin1Char('@') + QString::number(dpr)
+                      + QLatin1String("x.") + fi.suffix();
+}
+
+QList<int> StyleHelper::availableImageResolutions(const QString &fileName)
+{
+    QList<int> result;
+    const int maxResolutions = qApp->devicePixelRatio();
+    for (int i = 1; i <= maxResolutions; ++i)
+        if (QFile::exists(imageFileWithResolution(fileName, i)))
+            result.append(i);
+    return result;
+}
+
 } // namespace Utils
