@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -56,18 +51,6 @@
 #include <QToolTip>
 #include <QWheelEvent>
 
-// QByteArray::toLower() is broken, it stops at the first \0
-static void lower(QByteArray &ba)
-{
-    char *data = ba.data();
-    char *end = data + ba.size();
-    while (data != end) {
-        if (*data >= 0x41 && *data <= 0x5A)
-            *data += 0x20;
-        ++data;
-    }
-}
-
 static QByteArray calculateHexPattern(const QByteArray &pattern)
 {
     QByteArray result;
@@ -393,6 +376,7 @@ bool BinEditorWidget::save(QString *errorString, const QString &oldFileName, con
 void BinEditorWidget::setSizes(quint64 startAddr, int range, int blockSize)
 {
     int newBlockSize = blockSize;
+    QTC_ASSERT(blockSize, return);
     QTC_ASSERT((blockSize/m_bytesPerLine) * m_bytesPerLine == blockSize,
                blockSize = (blockSize/m_bytesPerLine + 1) * m_bytesPerLine);
     // Users can edit data in the range
@@ -562,7 +546,7 @@ int BinEditorWidget::dataIndexOf(const QByteArray &pattern, int from, bool caseS
         ::memcpy(b + trailing, data.constData(), m_blockSize);
 
         if (!caseSensitive)
-            ::lower(buffer);
+            buffer = buffer.toLower();
 
         int pos = matcher.indexIn(buffer, from - (block * m_blockSize) + trailing);
         if (pos >= 0)
@@ -595,7 +579,7 @@ int BinEditorWidget::dataLastIndexOf(const QByteArray &pattern, int from, bool c
         ::memcpy(b, data.constData(), m_blockSize);
 
         if (!caseSensitive)
-            ::lower(buffer);
+            buffer = buffer.toLower();
 
         int pos = buffer.lastIndexOf(pattern, from - (block * m_blockSize));
         if (pos >= 0)
@@ -618,7 +602,7 @@ int BinEditorWidget::find(const QByteArray &pattern_arg, int from,
     bool caseSensitiveSearch = (findFlags & QTextDocument::FindCaseSensitively);
 
     if (!caseSensitiveSearch)
-        ::lower(pattern);
+        pattern = pattern.toLower();
 
     bool backwards = (findFlags & QTextDocument::FindBackward);
     int found = backwards ? dataLastIndexOf(pattern, from, caseSensitiveSearch)
@@ -709,6 +693,16 @@ QString BinEditorWidget::addressString(quint64 address)
     return m_addressString;
 }
 
+static void paintCursorBorder(QPainter *painter, const QRect &cursorRect)
+{
+    painter->save();
+    QPen borderPen(Qt::red);
+    borderPen.setJoinStyle(Qt::MiterJoin);
+    painter->setPen(borderPen);
+    painter->drawRect(QRectF(cursorRect).adjusted(0.5, 0.5, -0.5, -0.5));
+    painter->restore();
+}
+
 void BinEditorWidget::paintEvent(QPaintEvent *e)
 {
     QPainter painter(viewport());
@@ -734,7 +728,7 @@ void BinEditorWidget::paintEvent(QPaintEvent *e)
         patternData = dataMid(patternOffset, m_numVisibleLines * m_bytesPerLine + (topLine*m_bytesPerLine - patternOffset));
         patternDataHex = patternData;
         if (!m_caseSensitiveSearch)
-            ::lower(patternData);
+            patternData = patternData.toLower();
     }
 
     int foundPatternAt = findPattern(patternData, patternDataHex, patternOffset, patternOffset, &matchLength);
@@ -881,10 +875,7 @@ void BinEditorWidget::paintEvent(QPaintEvent *e)
         if (cursor >= 0) {
             int w = fm.boundingRect(itemString.mid(cursor*3, 2)).width();
             QRect cursorRect(x + cursor * m_columnWidth, y - m_ascent, w + 1, m_lineHeight);
-            painter.save();
-            painter.setPen(Qt::red);
-            painter.drawRect(cursorRect.adjusted(0, 0, 0, -1));
-            painter.restore();
+            paintCursorBorder(&painter, cursorRect);
             if (m_hexCursor && m_cursorVisible) {
                 if (m_lowNibble)
                     cursorRect.adjust(fm.width(itemString.left(1)), 0, 0, 0);
@@ -923,17 +914,16 @@ void BinEditorWidget::paintEvent(QPaintEvent *e)
                              y-m_ascent,
                              fm.width(printable.at(cursor)),
                              m_lineHeight);
-            painter.save();
             if (m_hexCursor || !m_cursorVisible) {
-                painter.setPen(Qt::red);
-                painter.drawRect(cursorRect.adjusted(0, 0, 0, -1));
+                paintCursorBorder(&painter, cursorRect);
             } else {
+                painter.save();
                 painter.setClipRect(cursorRect);
                 painter.fillRect(cursorRect, Qt::red);
                 painter.setPen(Qt::white);
                 painter.drawText(text_x, y, printable);
+                painter.restore();
             }
-            painter.restore();
         }
     }
 }
@@ -1279,12 +1269,19 @@ void BinEditorWidget::keyPressEvent(QKeyEvent *e)
 
 
     MoveMode moveMode = e->modifiers() & Qt::ShiftModifier ? KeepAnchor : MoveAnchor;
+    bool ctrlPressed = e->modifiers() & Qt::ControlModifier;
     switch (e->key()) {
     case Qt::Key_Up:
-        setCursorPosition(m_cursorPosition - m_bytesPerLine, moveMode);
+        if (ctrlPressed)
+            verticalScrollBar()->triggerAction(QScrollBar::SliderSingleStepSub);
+        else
+            setCursorPosition(m_cursorPosition - m_bytesPerLine, moveMode);
         break;
     case Qt::Key_Down:
-        setCursorPosition(m_cursorPosition + m_bytesPerLine, moveMode);
+        if (ctrlPressed)
+            verticalScrollBar()->triggerAction(QScrollBar::SliderSingleStepAdd);
+        else
+            setCursorPosition(m_cursorPosition + m_bytesPerLine, moveMode);
         break;
     case Qt::Key_Right:
         setCursorPosition(m_cursorPosition + 1, moveMode);
@@ -1297,12 +1294,13 @@ void BinEditorWidget::keyPressEvent(QKeyEvent *e)
         int line = qMax(0, m_cursorPosition / m_bytesPerLine - verticalScrollBar()->value());
         verticalScrollBar()->triggerAction(e->key() == Qt::Key_PageUp ?
                                            QScrollBar::SliderPageStepSub : QScrollBar::SliderPageStepAdd);
-        setCursorPosition((verticalScrollBar()->value() + line) * m_bytesPerLine + m_cursorPosition % m_bytesPerLine, moveMode);
+        if (!ctrlPressed)
+            setCursorPosition((verticalScrollBar()->value() + line) * m_bytesPerLine + m_cursorPosition % m_bytesPerLine, moveMode);
     } break;
 
     case Qt::Key_Home: {
         int pos;
-        if (e->modifiers() & Qt::ControlModifier)
+        if (ctrlPressed)
             pos = 0;
         else
             pos = m_cursorPosition/m_bytesPerLine * m_bytesPerLine;
@@ -1310,7 +1308,7 @@ void BinEditorWidget::keyPressEvent(QKeyEvent *e)
     } break;
     case Qt::Key_End: {
         int pos;
-        if (e->modifiers() & Qt::ControlModifier)
+        if (ctrlPressed)
             pos = m_size;
         else
             pos = m_cursorPosition/m_bytesPerLine * m_bytesPerLine + 15;
@@ -1380,8 +1378,9 @@ void BinEditorWidget::copy(bool raw)
                              tr("You cannot copy more than 4 MB of binary data."));
         return;
     }
-    const QByteArray &data = dataMid(selStart, selectionLength);
+    QByteArray data = dataMid(selStart, selectionLength);
     if (raw) {
+        data.replace(0, ' ');
         QApplication::clipboard()->setText(QString::fromLatin1(data));
         return;
     }
@@ -1403,7 +1402,7 @@ void BinEditorWidget::highlightSearchResults(const QByteArray &pattern, QTextDoc
     m_searchPattern = pattern;
     m_caseSensitiveSearch = (findFlags & QTextDocument::FindCaseSensitively);
     if (!m_caseSensitiveSearch)
-        ::lower(m_searchPattern);
+        m_searchPattern = m_searchPattern.toLower();
     m_searchPatternHex = calculateHexPattern(pattern);
     viewport()->update();
 }
