@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,21 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -33,18 +29,22 @@
 
 #include <coreplugin/icore.h>
 #include <coreplugin/actionmanager/actionmanager.h>
-#include <coreplugin/coreconstants.h>
+#include <coreplugin/actionmanager/command.h>
+#include <coreplugin/coreicons.h>
 #include <coreplugin/icontext.h>
 #include <utils/qtcassert.h>
+#include <utils/styledbar.h>
 
-#include <QSettings>
+#include <QAction>
+#include <QComboBox>
+#include <QCoreApplication>
 #include <QDebug>
 #include <QFont>
-#include <QAction>
-#include <QToolButton>
-#include <QComboBox>
+#include <QLabel>
 #include <QScrollArea>
+#include <QSettings>
 #include <QStackedWidget>
+#include <QToolButton>
 
 static const char SETTINGSKEYSECTIONNAME[] = "SearchResults";
 static const char SETTINGSKEYEXPANDRESULTS[] = "ExpandResults";
@@ -76,46 +76,74 @@ namespace Internal {
 
     class SearchResultWindowPrivate : public QObject
     {
-        Q_OBJECT
+        Q_DECLARE_TR_FUNCTIONS(Core::SearchResultWindow)
     public:
-        SearchResultWindowPrivate(SearchResultWindow *window);
-        bool isSearchVisible() const;
-        int visibleSearchIndex() const;
+        SearchResultWindowPrivate(SearchResultWindow *window, QWidget *newSearchPanel);
+        bool isSearchVisible() const { return m_currentIndex > 0; }
+        int visibleSearchIndex() const { return m_currentIndex - 1; }
         void setCurrentIndex(int index, bool focus);
+        void setCurrentIndexWithFocus(int index) { setCurrentIndex(index, true); }
+        void moveWidgetToTop();
+        void popupRequested(bool focus);
+        void handleExpandCollapseToolButton(bool checked);
 
-        FindPlugin *m_plugin;
         SearchResultWindow *q;
         QList<Internal::SearchResultWidget *> m_searchResultWidgets;
         QToolButton *m_expandCollapseButton;
         QAction *m_expandCollapseAction;
         static const bool m_initiallyExpand = false;
         QWidget *m_spacer;
+        QLabel *m_historyLabel;
+        QWidget *m_spacer2;
         QComboBox *m_recentSearchesBox;
         QStackedWidget *m_widget;
         QList<SearchResult *> m_searchResults;
         int m_currentIndex;
         QFont m_font;
         SearchResultColor m_color;
+        int m_tabWidth;
 
-    public slots:
-        void setCurrentIndex(int index);
-        void moveWidgetToTop();
-        void popupRequested(bool focus);
     };
 
-    SearchResultWindowPrivate::SearchResultWindowPrivate(SearchResultWindow *window)
-        : q(window)
+    SearchResultWindowPrivate::SearchResultWindowPrivate(SearchResultWindow *window, QWidget *nsp) :
+        q(window),
+        m_expandCollapseButton(0),
+        m_expandCollapseAction(new QAction(tr("Expand All"), window)),
+        m_spacer(new QWidget),
+        m_historyLabel(new QLabel(tr("History:"))),
+        m_spacer2(new QWidget),
+        m_recentSearchesBox(new QComboBox),
+        m_widget(new QStackedWidget),
+        m_currentIndex(0),
+        m_tabWidth(8)
     {
-    }
+        m_spacer->setMinimumWidth(30);
+        m_spacer2->setMinimumWidth(5);
+        m_recentSearchesBox->setProperty("drawleftborder", true);
+        m_recentSearchesBox->setSizeAdjustPolicy(QComboBox::AdjustToContents);
+        m_recentSearchesBox->addItem(tr("New Search"));
+        connect(m_recentSearchesBox, static_cast<void (QComboBox::*)(int)>(&QComboBox::activated),
+                this, &SearchResultWindowPrivate::setCurrentIndexWithFocus);
 
-    bool SearchResultWindowPrivate::isSearchVisible() const
-    {
-        return m_currentIndex > 0;
-    }
+        m_widget->setWindowTitle(q->displayName());
+
+        auto newSearchArea = new InternalScrollArea(m_widget);
+        newSearchArea->setWidget(nsp);
+        newSearchArea->setFocusProxy(nsp);
+        m_widget->addWidget(newSearchArea);
+
+        m_expandCollapseButton = new QToolButton(m_widget);
+        m_expandCollapseButton->setAutoRaise(true);
+
+        m_expandCollapseAction->setCheckable(true);
+        m_expandCollapseAction->setIcon(Core::Icons::EXPAND.icon());
+        Command *cmd = ActionManager::registerAction(m_expandCollapseAction, "Find.ExpandAll");
+        cmd->setAttribute(Command::CA_UpdateText);
+        m_expandCollapseButton->setDefaultAction(cmd->action());
+
+        connect(m_expandCollapseAction, &QAction::toggled,
+                this, &SearchResultWindowPrivate::handleExpandCollapseToolButton);
 
-    int SearchResultWindowPrivate::visibleSearchIndex() const
-    {
-        return m_currentIndex - 1;
     }
 
     void SearchResultWindowPrivate::setCurrentIndex(int index, bool focus)
@@ -138,11 +166,6 @@ namespace Internal {
         q->navigateStateChanged();
     }
 
-    void SearchResultWindowPrivate::setCurrentIndex(int index)
-    {
-        setCurrentIndex(index, true/*focus*/);
-    }
-
     void SearchResultWindowPrivate::moveWidgetToTop()
     {
         SearchResultWidget *widget = qobject_cast<SearchResultWidget *>(sender());
@@ -183,10 +206,10 @@ namespace Internal {
         QTC_ASSERT(widget, return);
         int internalIndex = m_searchResultWidgets.indexOf(widget) + 1/*account for "new search" entry*/;
         setCurrentIndex(internalIndex, focus);
-        q->popup(focus ? Core::IOutputPane::ModeSwitch | Core::IOutputPane::WithFocus
-                       : Core::IOutputPane::NoModeSwitch);
+        q->popup(focus ? IOutputPane::ModeSwitch | IOutputPane::WithFocus
+                       : IOutputPane::NoModeSwitch);
     }
-}
+} // namespace Internal
 
 using namespace Core::Internal;
 
@@ -270,39 +293,9 @@ SearchResultWindow *SearchResultWindow::m_instance = 0;
     \internal
 */
 SearchResultWindow::SearchResultWindow(QWidget *newSearchPanel)
-    : d(new SearchResultWindowPrivate(this))
+    : d(new SearchResultWindowPrivate(this, newSearchPanel))
 {
     m_instance = this;
-
-    d->m_spacer = new QWidget;
-    d->m_spacer->setMinimumWidth(30);
-    d->m_recentSearchesBox = new QComboBox;
-    d->m_recentSearchesBox->setSizeAdjustPolicy(QComboBox::AdjustToContents);
-    d->m_recentSearchesBox->addItem(tr("New Search"));
-    connect(d->m_recentSearchesBox, SIGNAL(activated(int)), d, SLOT(setCurrentIndex(int)));
-
-    d->m_widget = new QStackedWidget;
-    d->m_widget->setWindowTitle(displayName());
-
-    InternalScrollArea *newSearchArea = new InternalScrollArea(d->m_widget);
-    newSearchArea->setWidget(newSearchPanel);
-    newSearchArea->setFocusProxy(newSearchPanel);
-    d->m_widget->addWidget(newSearchArea);
-    d->m_currentIndex = 0;
-
-    d->m_expandCollapseButton = new QToolButton(d->m_widget);
-    d->m_expandCollapseButton->setAutoRaise(true);
-
-    d->m_expandCollapseAction = new QAction(tr("Expand All"), this);
-    d->m_expandCollapseAction->setCheckable(true);
-    d->m_expandCollapseAction->setIcon(QIcon(QLatin1String(":/find/images/expand.png")));
-    Core::Command *cmd = Core::ActionManager::registerAction(
-            d->m_expandCollapseAction, "Find.ExpandAll",
-            Core::Context(Core::Constants::C_GLOBAL));
-    cmd->setAttribute(Core::Command::CA_UpdateText);
-    d->m_expandCollapseButton->setDefaultAction(cmd->action());
-
-    connect(d->m_expandCollapseAction, SIGNAL(toggled(bool)), this, SLOT(handleExpandCollapseToolButton(bool)));
     readSettings();
 }
 
@@ -347,7 +340,8 @@ QWidget *SearchResultWindow::outputWidget(QWidget *)
 */
 QList<QWidget*> SearchResultWindow::toolBarWidgets() const
 {
-    return QList<QWidget*>() << d->m_expandCollapseButton << d->m_spacer << d->m_recentSearchesBox;
+    return QList<QWidget*>() << d->m_expandCollapseButton << d->m_spacer
+                             << d->m_historyLabel << d->m_spacer2 << d->m_recentSearchesBox;
 }
 
 /*!
@@ -389,25 +383,28 @@ SearchResult *SearchResultWindow::startNewSearch(const QString &label,
             d->m_currentIndex = d->m_recentSearchesBox->count() - 1;
         }
     }
-    Internal::SearchResultWidget *widget = new Internal::SearchResultWidget;
+    auto widget = new SearchResultWidget;
     d->m_searchResultWidgets.prepend(widget);
     d->m_widget->insertWidget(1, widget);
-    connect(widget, SIGNAL(navigateStateChanged()), this, SLOT(navigateStateChanged()));
-    connect(widget, SIGNAL(restarted()), d, SLOT(moveWidgetToTop()));
-    connect(widget, SIGNAL(requestPopup(bool)), d, SLOT(popupRequested(bool)));
+    connect(widget, &SearchResultWidget::navigateStateChanged,
+            this, &SearchResultWindow::navigateStateChanged);
+    connect(widget, &SearchResultWidget::restarted,
+            d, &SearchResultWindowPrivate::moveWidgetToTop);
+    connect(widget, &SearchResultWidget::requestPopup,
+            d, &SearchResultWindowPrivate::popupRequested);
     widget->setTextEditorFont(d->m_font, d->m_color);
+    widget->setTabWidth(d->m_tabWidth);
     widget->setSupportPreserveCase(preserveCaseMode == PreserveCaseEnabled);
-    widget->setShowReplaceUI(searchOrSearchAndReplace != SearchOnly);
+    bool supportsReplace = searchOrSearchAndReplace != SearchOnly;
+    widget->setSupportsReplace(supportsReplace, supportsReplace ? cfgGroup : QString());
     widget->setAutoExpandResults(d->m_expandCollapseAction->isChecked());
     widget->setInfo(label, toolTip, searchTerm);
-    if (searchOrSearchAndReplace == SearchAndReplace)
-        widget->setDontAskAgainGroup(cfgGroup);
-    SearchResult *result = new SearchResult(widget);
+    auto result = new SearchResult(widget);
     d->m_searchResults.prepend(result);
     d->m_recentSearchesBox->insertItem(1, tr("%1 %2").arg(label, searchTerm));
     if (d->m_currentIndex > 0)
         ++d->m_currentIndex; // so setCurrentIndex still knows about the right "currentIndex" and its widget
-    d->setCurrentIndex(1);
+    d->setCurrentIndexWithFocus(1);
     return result;
 }
 
@@ -436,7 +433,10 @@ void SearchResultWindow::clearContents()
 */
 bool SearchResultWindow::hasFocus() const
 {
-    return d->m_widget->focusWidget() && d->m_widget->focusWidget()->hasFocus();
+    QWidget *widget = d->m_widget->focusWidget();
+    if (!widget)
+        return false;
+    return widget->window()->focusWidget() == widget;
 }
 
 /*!
@@ -484,26 +484,30 @@ void SearchResultWindow::setTextEditorFont(const QFont &font,
         widget->setTextEditorFont(font, color);
 }
 
+void SearchResultWindow::setTabWidth(int tabWidth)
+{
+    d->m_tabWidth = tabWidth;
+    foreach (Internal::SearchResultWidget *widget, d->m_searchResultWidgets)
+        widget->setTabWidth(tabWidth);
+}
+
 void SearchResultWindow::openNewSearchPanel()
 {
-    d->setCurrentIndex(0);
+    d->setCurrentIndexWithFocus(0);
     popup(IOutputPane::ModeSwitch  | IOutputPane::WithFocus | IOutputPane::EnsureSizeHint);
 }
 
-/*!
-    \internal
-*/
-void SearchResultWindow::handleExpandCollapseToolButton(bool checked)
+void SearchResultWindowPrivate::handleExpandCollapseToolButton(bool checked)
 {
-    if (!d->isSearchVisible())
+    if (!isSearchVisible())
         return;
-    d->m_searchResultWidgets.at(d->visibleSearchIndex())->setAutoExpandResults(checked);
+    m_searchResultWidgets.at(visibleSearchIndex())->setAutoExpandResults(checked);
     if (checked) {
-        d->m_expandCollapseAction->setText(tr("Collapse All"));
-        d->m_searchResultWidgets.at(d->visibleSearchIndex())->expandAll();
+        m_expandCollapseAction->setText(tr("Collapse All"));
+        m_searchResultWidgets.at(visibleSearchIndex())->expandAll();
     } else {
-        d->m_expandCollapseAction->setText(tr("Expand All"));
-        d->m_searchResultWidgets.at(d->visibleSearchIndex())->collapseAll();
+        m_expandCollapseAction->setText(tr("Expand All"));
+        m_searchResultWidgets.at(visibleSearchIndex())->collapseAll();
     }
 }
 
@@ -512,7 +516,7 @@ void SearchResultWindow::handleExpandCollapseToolButton(bool checked)
 */
 void SearchResultWindow::readSettings()
 {
-    QSettings *s = Core::ICore::settings();
+    QSettings *s = ICore::settings();
     s->beginGroup(QLatin1String(SETTINGSKEYSECTIONNAME));
     d->m_expandCollapseAction->setChecked(s->value(QLatin1String(SETTINGSKEYEXPANDRESULTS), d->m_initiallyExpand).toBool());
     s->endGroup();
@@ -523,7 +527,7 @@ void SearchResultWindow::readSettings()
 */
 void SearchResultWindow::writeSettings()
 {
-    QSettings *s = Core::ICore::settings();
+    QSettings *s = ICore::settings();
     s->beginGroup(QLatin1String(SETTINGSKEYSECTIONNAME));
     s->setValue(QLatin1String(SETTINGSKEYEXPANDRESULTS), d->m_expandCollapseAction->isChecked());
     s->endGroup();
@@ -589,18 +593,15 @@ bool SearchResultWindow::canNavigate() const
 SearchResult::SearchResult(SearchResultWidget *widget)
     : m_widget(widget)
 {
-    connect(widget, SIGNAL(activated(Core::SearchResultItem)),
-            this, SIGNAL(activated(Core::SearchResultItem)));
-    connect(widget, SIGNAL(replaceButtonClicked(QString,QList<Core::SearchResultItem>,bool)),
-            this, SIGNAL(replaceButtonClicked(QString,QList<Core::SearchResultItem>,bool)));
-    connect(widget, SIGNAL(cancelled()),
-            this, SIGNAL(cancelled()));
-    connect(widget, SIGNAL(paused(bool)),
-            this, SIGNAL(paused(bool)));
-    connect(widget, SIGNAL(visibilityChanged(bool)),
-            this, SIGNAL(visibilityChanged(bool)));
-    connect(widget, SIGNAL(searchAgainRequested()),
-            this, SIGNAL(searchAgainRequested()));
+    connect(widget, &SearchResultWidget::activated, this, &SearchResult::activated);
+    connect(widget, &SearchResultWidget::replaceButtonClicked,
+            this, &SearchResult::replaceButtonClicked);
+    connect(widget, &SearchResultWidget::cancelled, this, &SearchResult::cancelled);
+    connect(widget, &SearchResultWidget::paused, this, &SearchResult::paused);
+    connect(widget, &SearchResultWidget::visibilityChanged,
+            this, &SearchResult::visibilityChanged);
+    connect(widget, &SearchResultWidget::searchAgainRequested,
+            this, &SearchResult::searchAgainRequested);
 }
 
 /*!
