@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -46,11 +41,8 @@ namespace Core {
 class CORE_EXPORT Id
 {
 public:
-    enum { IdsPerPlugin = 10000, ReservedPlugins = 1000 };
-
     Id() : m_id(0) {}
-    Id(int uid) : m_id(uid) {}
-    Id(const char *name);
+    Id(const char *name); // Good to use.
 
     Id withSuffix(int suffix) const;
     Id withSuffix(const char *suffix) const;
@@ -69,20 +61,26 @@ public:
     bool operator<(Id id) const { return m_id < id.m_id; }
     bool operator>(Id id) const { return m_id > id.m_id; }
     bool alphabeticallyBefore(Id other) const;
-    int uniqueIdentifier() const { return m_id; }
-    static Id fromUniqueIdentifier(int uid) { return Id(uid); }
+
+    quintptr uniqueIdentifier() const { return m_id; } // Avoid.
     static Id fromString(const QString &str); // FIXME: avoid.
     static Id fromName(const QByteArray &ba); // FIXME: avoid.
     static Id fromSetting(const QVariant &variant); // Good to use.
-    static void registerId(int uid, const char *name);
+
+    static Id versionedId(const QByteArray &prefix, int major, int minor = -1);
+
+    static QSet<Id> fromStringList(const QStringList &list);
+    static QStringList toStringList(const QSet<Id> &ids);
 
 private:
     // Intentionally unimplemented
-    Id(const QLatin1String &);
-    int m_id;
+    Id(const QLatin1String &) = delete;
+    explicit Id(quintptr uid) : m_id(uid) {}
+
+    quintptr m_id;
 };
 
-inline uint qHash(Id id) { return id.uniqueIdentifier(); }
+inline uint qHash(Id id) { return static_cast<uint>(id.uniqueIdentifier()); }
 
 } // namespace Core
 
@@ -92,6 +90,7 @@ Q_DECLARE_METATYPE(QList<Core::Id>)
 QT_BEGIN_NAMESPACE
 QDataStream &operator<<(QDataStream &ds, Core::Id id);
 QDataStream &operator>>(QDataStream &ds, Core::Id &id);
+CORE_EXPORT QDebug operator<<(QDebug dbg, const Core::Id &id);
 QT_END_NAMESPACE
 
 #endif // CORE_ID_H
