@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -35,7 +30,8 @@
 #include "locatorwidget.h"
 #include "opendocumentsfilter.h"
 #include "filesystemfilter.h"
-#include "settingspage.h"
+#include "locatorsettingspage.h"
+#include "externaltoolsfilter.h"
 
 #include <coreplugin/coreplugin.h>
 #include <coreplugin/statusbarwidget.h>
@@ -44,11 +40,13 @@
 #include <coreplugin/icore.h>
 #include <coreplugin/actionmanager/actionmanager.h>
 #include <coreplugin/actionmanager/actioncontainer.h>
+#include <coreplugin/editormanager/editormanager.h>
+#include <coreplugin/editormanager/editormanager_p.h>
 #include <coreplugin/progressmanager/progressmanager.h>
 #include <coreplugin/progressmanager/futureprogress.h>
 #include <extensionsystem/pluginmanager.h>
 #include <utils/algorithm.h>
-#include <utils/QtConcurrentTools>
+#include <utils/mapreduce.h>
 #include <utils/qtcassert.h>
 
 #include <QSettings>
@@ -56,6 +54,10 @@
 #include <QFuture>
 #include <QAction>
 
+#ifdef Q_OS_OSX
+#include "spotlightlocatorfilter.h"
+#endif
+
 namespace Core {
 namespace Internal {
 
@@ -64,7 +66,7 @@ Locator::Locator()
 {
     m_corePlugin = 0;
     m_refreshTimer.setSingleShot(false);
-    connect(&m_refreshTimer, SIGNAL(timeout()), this, SLOT(refresh()));
+    connect(&m_refreshTimer, &QTimer::timeout, this, [this]() { refresh(); });
 }
 
 Locator::~Locator()
@@ -73,10 +75,12 @@ Locator::~Locator()
     m_corePlugin->removeObject(m_fileSystemFilter);
     m_corePlugin->removeObject(m_executeFilter);
     m_corePlugin->removeObject(m_settingsPage);
+    m_corePlugin->removeObject(m_externalToolsFilter);
     delete m_openDocumentsFilter;
     delete m_fileSystemFilter;
     delete m_executeFilter;
     delete m_settingsPage;
+    delete m_externalToolsFilter;
     qDeleteAll(m_customFilters);
 }
 
@@ -84,7 +88,7 @@ void Locator::initialize(CorePlugin *corePlugin, const QStringList &, QString *)
 {
     m_corePlugin = corePlugin;
 
-    m_settingsPage = new SettingsPage(this);
+    m_settingsPage = new LocatorSettingsPage(this);
     m_corePlugin->addObject(m_settingsPage);
 
     m_locatorWidget = new LocatorWidget(this);
@@ -96,14 +100,14 @@ void Locator::initialize(CorePlugin *corePlugin, const QStringList &, QString *)
     m_corePlugin->addAutoReleasedObject(view);
 
     QAction *action = new QAction(m_locatorWidget->windowIcon(), m_locatorWidget->windowTitle(), this);
-    Command *cmd = ActionManager::registerAction(action, "QtCreator.Locate",
-                                                             Context(Core::Constants::C_GLOBAL));
+    Command *cmd = ActionManager::registerAction(action, Constants::LOCATE);
     cmd->setDefaultKeySequence(QKeySequence(tr("Ctrl+K")));
-    connect(action, SIGNAL(triggered()), this, SLOT(openLocator()));
-    connect(cmd, SIGNAL(keySequenceChanged()), this, SLOT(updatePlaceholderText()));
+    connect(action, &QAction::triggered, this, &Locator::openLocator);
+    connect(cmd, &Command::keySequenceChanged,
+            this, [this, cmd]() { updatePlaceholderText(cmd); });
     updatePlaceholderText(cmd);
 
-    ActionContainer *mtools = ActionManager::actionContainer(Core::Constants::M_TOOLS);
+    ActionContainer *mtools = ActionManager::actionContainer(Constants::M_TOOLS);
     mtools->addAction(cmd);
 
     m_corePlugin->addObject(new LocatorManager(m_locatorWidget));
@@ -117,15 +121,19 @@ void Locator::initialize(CorePlugin *corePlugin, const QStringList &, QString *)
     m_executeFilter = new ExecuteFilter();
     m_corePlugin->addObject(m_executeFilter);
 
-    m_corePlugin->addAutoReleasedObject(new LocatorFiltersFilter(this, m_locatorWidget));
+    m_externalToolsFilter = new ExternalToolsFilter;
+    m_corePlugin->addObject(m_externalToolsFilter);
 
-    connect(ICore::instance(), SIGNAL(saveSettingsRequested()), this, SLOT(saveSettings()));
+    m_corePlugin->addAutoReleasedObject(new LocatorFiltersFilter(this, m_locatorWidget));
+#ifdef Q_OS_OSX
+    m_corePlugin->addAutoReleasedObject(new SpotlightLocatorFilter);
+#endif
+
+    connect(ICore::instance(), &ICore::saveSettingsRequested, this, &Locator::saveSettings);
 }
 
 void Locator::updatePlaceholderText(Command *command)
 {
-    if (!command)
-        command = qobject_cast<Command *>(sender());
     QTC_ASSERT(command, return);
     if (command->keySequence().isEmpty())
         m_locatorWidget->setPlaceholderText(tr("Type to locate"));
@@ -148,6 +156,14 @@ void Locator::extensionsInitialized()
         return first->id().alphabeticallyBefore(second->id());
     });
     setFilters(m_filters);
+
+    Command *openCommand = ActionManager::command(Constants::OPEN);
+    Command *locateCommand = ActionManager::command(Constants::LOCATE);
+    connect(openCommand, &Command::keySequenceChanged,
+            this, &Locator::updateEditorManagerPlaceholderText);
+    connect(locateCommand, &Command::keySequenceChanged,
+            this, &Locator::updateEditorManagerPlaceholderText);
+    updateEditorManagerPlaceholderText();
 }
 
 bool Locator::delayedInitialize()
@@ -177,6 +193,51 @@ void Locator::loadSettings()
     m_settingsInitialized = true;
 }
 
+void Locator::updateEditorManagerPlaceholderText()
+{
+    Command *openCommand = ActionManager::command(Constants::OPEN);
+    Command *locateCommand = ActionManager::command(Constants::LOCATE);
+    const QString placeholderText = tr("<html><body style=\"color:#909090; font-size:14px\">"
+          "<div align='center'>"
+          "<div style=\"font-size:20px\">Open a document</div>"
+          "<table><tr><td>"
+          "<hr/>"
+          "<div style=\"margin-top: 5px\">&bull; File > Open File or Project (%1)</div>"
+          "<div style=\"margin-top: 5px\">&bull; File > Recent Files</div>"
+          "<div style=\"margin-top: 5px\">&bull; Tools > Locate (%2) and</div>"
+          "<div style=\"margin-left: 1em\">- type to open file from any open project</div>"
+          "%4"
+          "%5"
+          "<div style=\"margin-left: 1em\">- type <code>%3&lt;space&gt;&lt;filename&gt;</code> to open file from file system</div>"
+          "<div style=\"margin-left: 1em\">- select one of the other filters for jumping to a location</div>"
+          "<div style=\"margin-top: 5px\">&bull; Drag and drop files here</div>"
+          "</td></tr></table>"
+          "</div>"
+          "</body></html>")
+         .arg(openCommand->keySequence().toString(QKeySequence::NativeText))
+         .arg(locateCommand->keySequence().toString(QKeySequence::NativeText))
+         .arg(m_fileSystemFilter->shortcutString());
+
+    QString classes;
+    // not nice, but anyhow
+    ILocatorFilter *classesFilter = Utils::findOrDefault(m_filters,
+                                                         Utils::equal(&ILocatorFilter::id,
+                                                                      Id("Classes")));
+    if (classesFilter)
+        classes = tr("<div style=\"margin-left: 1em\">- type <code>%1&lt;space&gt;&lt;pattern&gt;</code>"
+                     " to jump to a class definition</div>").arg(classesFilter->shortcutString());
+
+    QString methods;
+    // not nice, but anyhow
+    ILocatorFilter *methodsFilter = Utils::findOrDefault(m_filters, Utils::equal(&ILocatorFilter::id,
+                                                                                 Id("Methods")));
+    if (methodsFilter)
+        methods = tr("<div style=\"margin-left: 1em\">- type <code>%1&lt;space&gt;&lt;pattern&gt;</code>"
+                     " to jump to a function definition</div>").arg(methodsFilter->shortcutString());
+
+    EditorManagerPrivate::setPlaceholderText(placeholderText.arg(classes, methods));
+}
+
 void Locator::saveSettings()
 {
     if (m_settingsInitialized) {
@@ -222,6 +283,7 @@ QList<ILocatorFilter *> Locator::customFilters()
 void Locator::setFilters(QList<ILocatorFilter *> f)
 {
     m_filters = f;
+    updateEditorManagerPlaceholderText(); // possibly some shortcut changed
     m_locatorWidget->updateFilterList();
 }
 
@@ -250,10 +312,10 @@ void Locator::refresh(QList<ILocatorFilter *> filters)
 {
     if (filters.isEmpty())
         filters = m_filters;
-    QFuture<void> task = QtConcurrent::run(&ILocatorFilter::refresh, filters);
+    QFuture<void> task = Utils::map(filters, &ILocatorFilter::refresh);
     FutureProgress *progress =
-        ProgressManager::addTask(task, tr("Updating Locator Caches"), Core::Constants::TASK_INDEX);
-    connect(progress, SIGNAL(finished()), this, SLOT(saveSettings()));
+        ProgressManager::addTask(task, tr("Updating Locator Caches"), Constants::TASK_INDEX);
+    connect(progress, &FutureProgress::finished, this, &Locator::saveSettings);
 }
 
 } // namespace Internal
