@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -44,19 +39,28 @@
 #include "cpptoolsreuse.h"
 #include "cppprojectfile.h"
 #include "cpplocatordata.h"
+#include "cppincludesfilter.h"
+#include "cpptoolsbridge.h"
+#include "projectinfo.h"
+#include "cpptoolsbridgeqtcreatorimplementation.h"
 
 #include <coreplugin/actionmanager/actioncontainer.h>
 #include <coreplugin/actionmanager/actionmanager.h>
+#include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/coreconstants.h>
 #include <coreplugin/documentmanager.h>
 #include <coreplugin/icore.h>
+#include <coreplugin/idocument.h>
 #include <coreplugin/jsexpander.h>
 #include <coreplugin/vcsmanager.h>
 #include <cppeditor/cppeditorconstants.h>
+#include <projectexplorer/project.h>
+#include <projectexplorer/projecttree.h>
 
 #include <utils/fileutils.h>
 #include <utils/hostosinfo.h>
 #include <utils/macroexpander.h>
+#include <utils/mimetypes/mimedatabase.h>
 #include <utils/qtcassert.h>
 
 #include <QtPlugin>
@@ -82,12 +86,13 @@ CppToolsPlugin::CppToolsPlugin()
     , m_codeModelSettings(new CppCodeModelSettings)
 {
     m_instance = this;
+    auto bridgeImplementation = std::unique_ptr<CppToolsBridgeQtCreatorImplementation>(new CppToolsBridgeQtCreatorImplementation);
+    CppToolsBridge::setCppToolsBridgeImplementation(std::move(bridgeImplementation));
 }
 
 CppToolsPlugin::~CppToolsPlugin()
 {
     m_instance = 0;
-    delete CppModelManager::instance();
 }
 
 CppToolsPlugin *CppToolsPlugin::instance()
@@ -130,12 +135,15 @@ const QStringList &CppToolsPlugin::sourcePrefixes()
     return m_instance->m_fileSettings->sourcePrefixes;
 }
 
-
 bool CppToolsPlugin::initialize(const QStringList &arguments, QString *error)
 {
     Q_UNUSED(arguments)
     Q_UNUSED(error)
 
+    Utils::MimeDatabase::addMimeTypes(QLatin1String(":/cpptools/CppTools.mimetypes.xml"));
+
+    CppModelManager::instance()->setParent(this);
+
     m_settings = new CppToolsSettings(this); // force registration of cpp tools settings
 
     // Objects
@@ -147,18 +155,21 @@ bool CppToolsPlugin::initialize(const QStringList &arguments, QString *error)
         modelManager->updateSourceFiles(files.toSet());
     });
 
+    m_codeModelSettings->fromSettings(ICore::settings());
+
     JsExpander::registerQObjectForJs(QLatin1String("Cpp"), new CppToolsJsExtension);
 
     CppLocatorData *locatorData = new CppLocatorData;
-    connect(modelManager, SIGNAL(documentUpdated(CPlusPlus::Document::Ptr)),
-            locatorData, SLOT(onDocumentUpdated(CPlusPlus::Document::Ptr)));
+    connect(modelManager, &CppModelManager::documentUpdated,
+            locatorData, &CppLocatorData::onDocumentUpdated);
 
-    connect(modelManager, SIGNAL(aboutToRemoveFiles(QStringList)),
-            locatorData, SLOT(onAboutToRemoveFiles(QStringList)));
+    connect(modelManager, &CppModelManager::aboutToRemoveFiles,
+            locatorData, &CppLocatorData::onAboutToRemoveFiles);
 
     addAutoReleasedObject(locatorData);
     addAutoReleasedObject(new CppLocatorFilter(locatorData));
     addAutoReleasedObject(new CppClassesFilter(locatorData));
+    addAutoReleasedObject(new CppIncludesFilter);
     addAutoReleasedObject(new CppFunctionsFilter(locatorData));
     addAutoReleasedObject(new CppCurrentDocumentFilter(modelManager, m_stringTable));
     addAutoReleasedObject(new CppFileSettingsPage(m_fileSettings));
@@ -181,7 +192,8 @@ bool CppToolsPlugin::initialize(const QStringList &arguments, QString *error)
     Command *command = ActionManager::registerAction(switchAction, Constants::SWITCH_HEADER_SOURCE, context, true);
     command->setDefaultKeySequence(QKeySequence(Qt::Key_F4));
     mcpptools->addAction(command);
-    connect(switchAction, SIGNAL(triggered()), this, SLOT(switchHeaderSource()));
+    connect(switchAction, &QAction::triggered,
+            this, &CppToolsPlugin::switchHeaderSource);
 
     QAction *openInNextSplitAction = new QAction(tr("Open Corresponding Header/Source in Next Split"), this);
     command = ActionManager::registerAction(openInNextSplitAction, Constants::OPEN_HEADER_SOURCE_IN_NEXT_SPLIT, context, true);
@@ -189,7 +201,8 @@ bool CppToolsPlugin::initialize(const QStringList &arguments, QString *error)
                                                 ? tr("Meta+E, F4")
                                                 : tr("Ctrl+E, F4")));
     mcpptools->addAction(command);
-    connect(openInNextSplitAction, SIGNAL(triggered()), this, SLOT(switchHeaderSourceInNextSplit()));
+    connect(openInNextSplitAction, &QAction::triggered,
+            this, &CppToolsPlugin::switchHeaderSourceInNextSplit);
 
     Utils::MacroExpander *expander = Utils::globalMacroExpander();
     expander->registerVariable("Cpp:LicenseTemplate",
@@ -209,7 +222,6 @@ void CppToolsPlugin::extensionsInitialized()
     m_fileSettings->fromSettings(ICore::settings());
     if (!m_fileSettings->applySuffixesToMimeDB())
         qWarning("Unable to apply cpp suffixes to mime database (cpp mime types not found).\n");
-    m_codeModelSettings->fromSettings(ICore::settings());
 }
 
 ExtensionSystem::IPlugin::ShutdownFlag CppToolsPlugin::aboutToShutdown()
@@ -235,7 +247,7 @@ void CppToolsPlugin::switchHeaderSource()
 void CppToolsPlugin::switchHeaderSourceInNextSplit()
 {
     QString otherFile = correspondingHeaderOrSource(
-                EditorManager::currentDocument()->filePath());
+                EditorManager::currentDocument()->filePath().toString());
     if (!otherFile.isEmpty())
         EditorManager::openEditor(otherFile, Id(), EditorManager::OpenInOtherSplit);
 }
@@ -255,7 +267,7 @@ static QStringList findFilesInProject(const QString &name,
     const QStringList::const_iterator pcend = projectFiles.constEnd();
     QStringList candidateList;
     for (QStringList::const_iterator it = projectFiles.constBegin(); it != pcend; ++it) {
-        if (it->endsWith(pattern))
+        if (it->endsWith(pattern, Utils::HostOsInfo::fileNameCaseSensitivity()))
             candidateList.append(*it);
     }
     return candidateList;
@@ -265,24 +277,25 @@ static QStringList findFilesInProject(const QString &name,
 // source belonging to a header and vice versa
 static QStringList matchingCandidateSuffixes(ProjectFile::Kind kind)
 {
+    Utils::MimeDatabase mdb;
     switch (kind) {
      // Note that C/C++ headers are undistinguishable
     case ProjectFile::CHeader:
     case ProjectFile::CXXHeader:
     case ProjectFile::ObjCHeader:
     case ProjectFile::ObjCXXHeader:
-        return MimeDatabase::findByType(QLatin1String(Constants::C_SOURCE_MIMETYPE)).suffixes()
-                + MimeDatabase::findByType(QLatin1String(Constants::CPP_SOURCE_MIMETYPE)).suffixes()
-                + MimeDatabase::findByType(QLatin1String(Constants::OBJECTIVE_C_SOURCE_MIMETYPE)).suffixes()
-                + MimeDatabase::findByType(QLatin1String(Constants::OBJECTIVE_CPP_SOURCE_MIMETYPE)).suffixes();
+        return mdb.mimeTypeForName(QLatin1String(Constants::C_SOURCE_MIMETYPE)).suffixes()
+                + mdb.mimeTypeForName(QLatin1String(Constants::CPP_SOURCE_MIMETYPE)).suffixes()
+                + mdb.mimeTypeForName(QLatin1String(Constants::OBJECTIVE_C_SOURCE_MIMETYPE)).suffixes()
+                + mdb.mimeTypeForName(QLatin1String(Constants::OBJECTIVE_CPP_SOURCE_MIMETYPE)).suffixes();
     case ProjectFile::CSource:
     case ProjectFile::ObjCSource:
-        return MimeDatabase::findByType(QLatin1String(Constants::C_HEADER_MIMETYPE)).suffixes();
+        return mdb.mimeTypeForName(QLatin1String(Constants::C_HEADER_MIMETYPE)).suffixes();
     case ProjectFile::CXXSource:
     case ProjectFile::ObjCXXSource:
     case ProjectFile::CudaSource:
     case ProjectFile::OpenCLSource:
-        return MimeDatabase::findByType(QLatin1String(Constants::CPP_HEADER_MIMETYPE)).suffixes();
+        return mdb.mimeTypeForName(QLatin1String(Constants::CPP_HEADER_MIMETYPE)).suffixes();
     default:
         return QStringList();
     }
@@ -331,12 +344,17 @@ static QStringList baseDirWithAllDirectories(const QDir &baseDir, const QStringL
     return result;
 }
 
-static int commonStringLength(const QString &s1, const QString &s2)
+static int commonFilePathLength(const QString &s1, const QString &s2)
 {
     int length = qMin(s1.length(), s2.length());
     for (int i = 0; i < length; ++i)
-        if (s1[i] != s2[i])
-            return i;
+        if (Utils::HostOsInfo::fileNameCaseSensitivity() == Qt::CaseSensitive) {
+            if (s1[i] != s2[i])
+                return i;
+        } else {
+            if (s1[i].toLower() != s2[i].toLower())
+                return i;
+        }
     return length;
 }
 
@@ -351,7 +369,7 @@ static QString correspondingHeaderOrSourceInProject(const QFileInfo &fileInfo,
         const QStringList projectFiles = findFilesInProject(candidateFileName, project);
         // Find the file having the most common path with fileName
         foreach (const QString &projectFile, projectFiles) {
-            int value = commonStringLength(filePath, projectFile);
+            int value = commonFilePathLength(filePath, projectFile);
             if (value > compareValue) {
                 compareValue = value;
                 bestFileName = projectFile;
@@ -431,7 +449,7 @@ QString correspondingHeaderOrSource(const QString &fileName, bool *wasHeader)
     }
 
     // Find files in the current project
-    ProjectExplorer::Project *currentProject = ProjectExplorer::ProjectExplorerPlugin::currentProject();
+    ProjectExplorer::Project *currentProject = ProjectExplorer::ProjectTree::currentProject();
     if (currentProject) {
         const QString path = correspondingHeaderOrSourceInProject(fi, candidateFileNames,
                                                                   currentProject);
