@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -40,6 +35,8 @@
 namespace Debugger {
 namespace Internal {
 
+class DebuggerEngine;
+
 enum RegisterColumns
 {
     RegisterNameColumn,
@@ -71,6 +68,7 @@ enum RegisterFormat
     CharacterFormat,
     HexadecimalFormat,
     DecimalFormat,
+    SignedDecimalFormat,
     OctalFormat,
     BinaryFormat
 };
@@ -79,11 +77,17 @@ class RegisterValue
 {
 public:
     RegisterValue() { known = false; v.u64[1] = v.u64[0] = 0; }
-    void operator=(const QByteArray &ba);
     bool operator==(const RegisterValue &other);
     bool operator!=(const RegisterValue &other) { return !operator==(other); }
-    QByteArray toByteArray(RegisterKind kind, int size, RegisterFormat format) const;
+
+    void fromByteArray(const QByteArray &ba, RegisterFormat format);
+    QByteArray toByteArray(RegisterKind kind, int size, RegisterFormat format,
+                           bool forEdit = false) const;
+
     RegisterValue subValue(int size, int index) const;
+    void setSubValue(int size, int index, RegisterValue subValue);
+
+    void shiftOneDigit(uint digit, RegisterFormat format);
 
     union {
         quint8  u8[16];
@@ -119,9 +123,10 @@ class RegisterHandler : public Utils::TreeModel
     Q_OBJECT
 
 public:
-    RegisterHandler();
+    explicit RegisterHandler(DebuggerEngine *engine);
 
     QAbstractItemModel *model() { return this; }
+    DebuggerEngine *engine() const { return m_engine; }
 
     void updateRegister(const Register &reg);
 
@@ -134,6 +139,7 @@ signals:
 
 private:
     QHash<QByteArray, RegisterItem *> m_registerByName;
+    DebuggerEngine * const m_engine;
 };
 
 } // namespace Internal
