@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -42,14 +37,15 @@
 #include <coreplugin/editortoolbar.h>
 #include <coreplugin/designmode.h>
 #include <coreplugin/coreconstants.h>
+#include <coreplugin/dialogs/ioptionspage.h>
 #include <coreplugin/icore.h>
 #include <coreplugin/infobar.h>
 #include <coreplugin/helpmanager.h>
 #include <coreplugin/actionmanager/actionmanager.h>
 #include <coreplugin/actionmanager/actioncontainer.h>
+#include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/modemanager.h>
 #include <coreplugin/minisplitter.h>
-#include <coreplugin/mimedatabase.h>
 #include <coreplugin/outputpane.h>
 #include <utils/qtcassert.h>
 
@@ -67,6 +63,7 @@
 #include <QApplication>
 #include <QCursor>
 #include <QDockWidget>
+#include <QMenu>
 #include <QMessageBox>
 #include <QKeySequence>
 #include <QPrintDialog>
@@ -118,9 +115,8 @@ class DesignerXmlEditorWidget : public TextEditor::TextEditorWidget
 public:
     DesignerXmlEditorWidget() {}
 
-    void finalizeInitialization()
+    void finalizeInitialization() override
     {
-        setupAsPlainEditor();
         setReadOnly(true);
     }
 };
@@ -130,9 +126,10 @@ class FormWindowEditorFactory : public TextEditor::TextEditorFactory
 public:
     FormWindowEditorFactory()
     {
-        setId(Designer::Constants::K_DESIGNER_XML_EDITOR_ID);
+        setId(K_DESIGNER_XML_EDITOR_ID);
         setEditorCreator([]() { return new FormWindowEditor; });
         setEditorWidgetCreator([]() { return new Internal::DesignerXmlEditorWidget; });
+        setUseGenericHighlighter(true);
         setDuplicatedSupported(false);
     }
 
@@ -168,42 +165,42 @@ public:
 
     void setupActions();
     void setupViewActions();
-    void addDockViewAction(Core::ActionContainer *viewMenu,
+    void addDockViewAction(ActionContainer *viewMenu,
                            int index,
-                           const Core::Context &context,
-                           const QString &title, Core::Id id);
+                           const Context &context,
+                           const QString &title, Id id);
 
-    Core::ActionContainer *createPreviewStyleMenu(QActionGroup *actionGroup);
+    ActionContainer *createPreviewStyleMenu(QActionGroup *actionGroup);
 
     void critical(const QString &errorMessage);
-    void bindShortcut(Core::Command *command, QAction *action);
+    void bindShortcut(Command *command, QAction *action);
     QAction *createEditModeAction(QActionGroup *ag,
-                                  const Core::Context &context,
-                                  Core::ActionContainer *medit,
+                                  const Context &context,
+                                  ActionContainer *medit,
                                   const QString &actionName,
-                                  Core::Id id,
+                                  Id id,
                                   int toolNumber,
                                   const QString &iconName = QString(),
                                   const QString &keySequence = QString());
-    Core::Command *addToolAction(QAction *a,
-                                 const Core::Context &context, Core::Id id,
-                                 Core::ActionContainer *c1, const QString &keySequence = QString(),
-                                 Core::Id groupId = Core::Id());
+    Command *addToolAction(QAction *a,
+                                 const Context &context, Id id,
+                                 ActionContainer *c1, const QString &keySequence = QString(),
+                                 Id groupId = Id());
     QToolBar *createEditorToolBar() const;
-    Core::IEditor *createEditor();
+    IEditor *createEditor();
 
 public:
     QDesignerFormEditorInterface *m_formeditor;
-    QDesignerIntegrationInterface *m_integration;
+    QtCreatorIntegration *m_integration;
     QDesignerFormWindowManagerInterface *m_fwm;
     FormEditorW::InitializationStage m_initStage;
 
-    QWidget *m_designerSubWindows[Designer::Constants::DesignerSubWindowCount];
+    QWidget *m_designerSubWindows[DesignerSubWindowCount];
 
     QAction *m_lockAction;
     QAction *m_resetLayoutAction;
 
-    QList<Core::IOptionsPage *> m_settingsPages;
+    QList<IOptionsPage *> m_settingsPages;
     QActionGroup *m_actionGroupEditMode;
     QAction *m_actionPrint;
     QAction *m_actionPreview;
@@ -213,17 +210,17 @@ public:
     QSignalMapper m_shortcutMapper;
 
     DesignerContext *m_context;
-    Core::Context m_contexts;
+    Context m_contexts;
 
-    QList<Core::Id> m_toolActionIds;
+    QList<Id> m_toolActionIds;
     QWidget *m_modeWidget;
     EditorWidget *m_editorWidget;
-    Core::DesignMode *m_designMode;
+    DesignMode *m_designMode;
 
     QWidget *m_editorToolBar;
-    Core::EditorToolBar *m_toolBar;
+    EditorToolBar *m_toolBar;
 
-    QMap<Core::Command *, QAction *> m_commandToDesignerAction;
+    QMap<Command *, QAction *> m_commandToDesignerAction;
     FormWindowEditorFactory *m_xmlEditorFactory;
 };
 
@@ -254,7 +251,7 @@ FormEditorData::FormEditorData() :
     QTC_ASSERT(!d, return);
     d = this;
 
-    qFill(m_designerSubWindows, m_designerSubWindows + Designer::Constants::DesignerSubWindowCount,
+    qFill(m_designerSubWindows, m_designerSubWindows + DesignerSubWindowCount,
           static_cast<QWidget *>(0));
 
     m_formeditor->setTopLevel(ICore::mainWindow());
@@ -263,7 +260,7 @@ FormEditorData::FormEditorData() :
     m_fwm = m_formeditor->formWindowManager();
     QTC_ASSERT(m_fwm, return);
 
-    m_contexts.add(Designer::Constants::C_FORMEDITOR);
+    m_contexts.add(C_FORMEDITOR);
 
     setupActions();
 
@@ -384,8 +381,9 @@ void FormEditorData::fullInit()
     m_integration = new QtCreatorIntegration(m_formeditor, m_instance);
     m_formeditor->setIntegration(m_integration);
     // Connect Qt Designer help request to HelpManager.
-    QObject::connect(m_integration, SIGNAL(creatorHelpRequested(QUrl)),
-        HelpManager::instance(), SLOT(handleHelpRequest(QUrl)));
+    QObject::connect(m_integration, &QtCreatorIntegration::creatorHelpRequested,
+                     HelpManager::instance(),
+                     [](const QUrl &url) { HelpManager::instance()->handleHelpRequest(url, HelpManager::HelpModeAlways); });
 
     /**
      * This will initialize our TabOrder, Signals and slots and Buddy editors.
@@ -456,7 +454,7 @@ void FormEditorData::fullInit()
 
 void FormEditorData::initDesignerSubWindows()
 {
-    qFill(m_designerSubWindows, m_designerSubWindows + Designer::Constants::DesignerSubWindowCount, static_cast<QWidget*>(0));
+    qFill(m_designerSubWindows, m_designerSubWindows + DesignerSubWindowCount, static_cast<QWidget*>(0));
 
     QDesignerWidgetBoxInterface *wb = QDesignerComponents::createWidgetBox(m_formeditor, 0);
     wb->setWindowTitle(tr("Widget Box"));
@@ -489,7 +487,7 @@ void FormEditorData::initDesignerSubWindows()
     m_initStage = FormEditorW::SubwindowsInitialized;
 }
 
-QList<Core::IOptionsPage *> FormEditorW::optionsPages()
+QList<IOptionsPage *> FormEditorW::optionsPages()
 {
     return d->m_settingsPages;
 }
@@ -738,9 +736,11 @@ void FormEditorData::critical(const QString &errorMessage)
 // Apply the command shortcut to the action and connects to the command's keySequenceChanged signal
 void FormEditorData::bindShortcut(Command *command, QAction *action)
 {
+    typedef void (QSignalMapper::*SignalMapperVoidSlot)();
+
     m_commandToDesignerAction.insert(command, action);
-    QObject::connect(command, SIGNAL(keySequenceChanged()),
-                     &m_shortcutMapper, SLOT(map()));
+    QObject::connect(command, &Command::keySequenceChanged,
+                     &m_shortcutMapper, static_cast<SignalMapperVoidSlot>(&QSignalMapper::map));
     m_shortcutMapper.setMapping(command, command);
     updateShortcut(command);
 }
@@ -760,7 +760,7 @@ QAction *FormEditorData::createEditModeAction(QActionGroup *ag,
     if (!iconName.isEmpty())
          rc->setIcon(designerIcon(iconName));
     Command *command = ActionManager::registerAction(rc, id, context);
-    command->setAttribute(Core::Command::CA_Hide);
+    command->setAttribute(Command::CA_Hide);
     if (!keySequence.isEmpty())
         command->setDefaultKeySequence(QKeySequence(keySequence));
     bindShortcut(command, rc);
@@ -773,7 +773,7 @@ QAction *FormEditorData::createEditModeAction(QActionGroup *ag,
 // Create a tool action
 Command *FormEditorData::addToolAction(QAction *a, const Context &context, Id id,
                                        ActionContainer *c1, const QString &keySequence,
-                                       Core::Id groupId)
+                                       Id groupId)
 {
     Command *command = ActionManager::registerAction(a, id, context);
     if (!keySequence.isEmpty())
@@ -784,7 +784,7 @@ Command *FormEditorData::addToolAction(QAction *a, const Context &context, Id id
     return command;
 }
 
-Core::IEditor *FormEditorData::createEditor()
+IEditor *FormEditorData::createEditor()
 {
     if (Designer::Constants::Internal::debug)
         qDebug() << "FormEditorW::createEditor";
@@ -801,7 +801,7 @@ Core::IEditor *FormEditorData::createEditor()
     m_toolBar->addEditor(formWindowEditor);
 
     if (formWindowEditor) {
-        Core::InfoBarEntry info(Core::Id(Constants::INFO_READ_ONLY),
+        InfoBarEntry info(Id(Constants::INFO_READ_ONLY),
                                 tr("This file can only be edited in <b>Design</b> mode."));
         info.setCustomButtonInfo(tr("Switch Mode"), []() { ModeManager::activateMode(Core::Constants::MODE_DESIGN); });
         formWindowEditor->document()->infoBar()->addInfo(info);
