@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -44,12 +39,7 @@
 namespace Perforce {
 namespace Internal {
 
-PerforceChecker::PerforceChecker(QObject *parent) :
-    QObject(parent),
-    m_timeOutMS(-1),
-    m_timedOut(false),
-    m_useOverideCursor(false),
-    m_isOverrideCursor(false)
+PerforceChecker::PerforceChecker(QObject *parent) : QObject(parent)
 {
     connect(&m_process, static_cast<void (QProcess::*)(QProcess::ProcessError)>(&QProcess::error),
             this, &PerforceChecker::slotError);
@@ -69,6 +59,11 @@ bool PerforceChecker::isRunning() const
     return m_process.state() == QProcess::Running;
 }
 
+bool PerforceChecker::waitForFinished(int msec)
+{
+    return m_process.waitForFinished(msec);
+}
+
 void PerforceChecker::resetOverrideCursor()
 {
     if (m_isOverrideCursor) {
@@ -77,7 +72,7 @@ void PerforceChecker::resetOverrideCursor()
     }
 }
 
-void PerforceChecker::start(const QString &binary,
+void PerforceChecker::start(const QString &binary, const QString &workingDirectory,
                             const QStringList &basicArgs,
                             int timeoutMS)
 {
@@ -92,13 +87,17 @@ void PerforceChecker::start(const QString &binary,
     m_binary = binary;
     QStringList args = basicArgs;
     args << QLatin1String("client") << QLatin1String("-o");
+
+    if (!workingDirectory.isEmpty())
+        m_process.setWorkingDirectory(workingDirectory);
+
     m_process.start(m_binary, args);
     m_process.closeWriteChannel();
     // Timeout handling
     m_timeOutMS = timeoutMS;
     m_timedOut = false;
     if (timeoutMS > 0)
-        QTimer::singleShot(m_timeOutMS, this, SLOT(slotTimeOut()));
+        QTimer::singleShot(m_timeOutMS, this, &PerforceChecker::slotTimeOut);
     // Cursor
     if (m_useOverideCursor) {
         m_isOverrideCursor = true;
@@ -112,7 +111,7 @@ void PerforceChecker::slotTimeOut()
         return;
     m_timedOut = true;
     Utils::SynchronousProcess::stopProcess(m_process);
-    emitFailed(tr("\"%1\" timed out after %2ms.").
+    emitFailed(tr("\"%1\" timed out after %2 ms.").
                arg(m_binary).arg(m_timeOutMS));
 }
 
