@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -69,6 +64,19 @@ static const char cxx11FlagsKeyC[] = "ProjectExplorer.CustomToolChain.Cxx11Flags
 static const char mkspecsKeyC[] = "ProjectExplorer.CustomToolChain.Mkspecs";
 static const char outputParserKeyC[] = "ProjectExplorer.CustomToolChain.OutputParser";
 static const char errorPatternKeyC[] = "ProjectExplorer.CustomToolChain.ErrorPattern";
+static const char errorLineNumberCapKeyC[] = "ProjectExplorer.CustomToolChain.ErrorLineNumberCap";
+static const char errorFileNameCapKeyC[] = "ProjectExplorer.CustomToolChain.ErrorFileNameCap";
+static const char errorMessageCapKeyC[] = "ProjectExplorer.CustomToolChain.ErrorMessageCap";
+static const char errorChannelKeyC[] = "ProjectExplorer.CustomToolChain.ErrorChannel";
+static const char errorExampleKeyC[] = "ProjectExplorer.CustomToolChain.ErrorExample";
+static const char warningPatternKeyC[] = "ProjectExplorer.CustomToolChain.WarningPattern";
+static const char warningLineNumberCapKeyC[] = "ProjectExplorer.CustomToolChain.WarningLineNumberCap";
+static const char warningFileNameCapKeyC[] = "ProjectExplorer.CustomToolChain.WarningFileNameCap";
+static const char warningMessageCapKeyC[] = "ProjectExplorer.CustomToolChain.WarningMessageCap";
+static const char warningChannelKeyC[] = "ProjectExplorer.CustomToolChain.WarningChannel";
+static const char warningExampleKeyC[] = "ProjectExplorer.CustomToolChain.WarningExample";
+
+// TODO Creator 4.1: remove (added in 3.7 for compatibility)
 static const char lineNumberCapKeyC[] = "ProjectExplorer.CustomToolChain.LineNumberCap";
 static const char fileNameCapKeyC[] = "ProjectExplorer.CustomToolChain.FileNameCap";
 static const char messageCapKeyC[] = "ProjectExplorer.CustomToolChain.MessageCap";
@@ -78,28 +86,10 @@ static const char messageCapKeyC[] = "ProjectExplorer.CustomToolChain.MessageCap
 // --------------------------------------------------------------------------
 
 CustomToolChain::CustomToolChain(Detection d) :
-    ToolChain(QLatin1String(Constants::CUSTOM_TOOLCHAIN_ID), d),
+    ToolChain(Constants::CUSTOM_TOOLCHAIN_TYPEID, d),
     m_outputParser(Gcc)
 { }
 
-CustomToolChain::CustomToolChain(const QString &id, Detection d) :
-    ToolChain(id, d)
-{ }
-
-CustomToolChain::CustomToolChain(const CustomToolChain &tc) :
-    ToolChain(tc),
-    m_compilerCommand(tc.m_compilerCommand),
-    m_makeCommand(tc.m_makeCommand),
-    m_targetAbi(tc.m_targetAbi),
-    m_predefinedMacros(tc.m_predefinedMacros),
-    m_systemHeaderPaths(tc.m_systemHeaderPaths)
-{ }
-
-QString CustomToolChain::type() const
-{
-    return QLatin1String("custom");
-}
-
 QString CustomToolChain::typeDisplayName() const
 {
     return Internal::CustomToolChainFactory::tr("Custom");
@@ -161,10 +151,10 @@ ToolChain::CompilerFlags CustomToolChain::compilerFlags(const QStringList &cxxfl
     return NoFlags;
 }
 
-ToolChain::WarningFlags CustomToolChain::warningFlags(const QStringList &cxxflags) const
+WarningFlags CustomToolChain::warningFlags(const QStringList &cxxflags) const
 {
     Q_UNUSED(cxxflags);
-    return WarningFlags(WarningsDefault);
+    return WarningFlags::Default;
 }
 
 const QStringList &CustomToolChain::rawPredefinedMacros() const
@@ -177,7 +167,7 @@ void CustomToolChain::setPredefinedMacros(const QStringList &list)
     m_predefinedMacros = list;
 }
 
-QList<HeaderPath> CustomToolChain::systemHeaderPaths(const QStringList &cxxFlags, const Utils::FileName &) const
+QList<HeaderPath> CustomToolChain::systemHeaderPaths(const QStringList &cxxFlags, const FileName &) const
 {
     QList<HeaderPath> flagHeaderPaths;
     foreach (const QString &cxxFlag, cxxFlags) {
@@ -199,7 +189,7 @@ void CustomToolChain::addToEnvironment(Environment &env) const
     }
 }
 
-QList<FileName> CustomToolChain::suggestedMkspecList() const
+FileNameList CustomToolChain::suggestedMkspecList() const
 {
     return m_mkspecs;
 }
@@ -251,7 +241,7 @@ void CustomToolChain::setMakeCommand(const FileName &path)
     toolChainUpdated();
 }
 
-QString CustomToolChain::makeCommand(const Utils::Environment &) const
+QString CustomToolChain::makeCommand(const Environment &) const
 {
     return m_makeCommand.toString();
 }
@@ -272,7 +262,7 @@ const QStringList &CustomToolChain::cxx11Flags() const
 void CustomToolChain::setMkspecs(const QString &specs)
 {
     m_mkspecs = Utils::transform(specs.split(QLatin1Char(',')),
-                                 [](QString fn) { return Utils::FileName::fromString(fn); });
+                                 [](QString fn) { return FileName::fromString(fn); });
 }
 
 QString CustomToolChain::mkspecs() const
@@ -300,10 +290,18 @@ QVariantMap CustomToolChain::toMap() const
     data.insert(QLatin1String(cxx11FlagsKeyC), m_cxx11Flags);
     data.insert(QLatin1String(mkspecsKeyC), mkspecs());
     data.insert(QLatin1String(outputParserKeyC), m_outputParser);
-    data.insert(QLatin1String(errorPatternKeyC), m_customParserSettings.errorPattern);
-    data.insert(QLatin1String(fileNameCapKeyC), m_customParserSettings.fileNameCap);
-    data.insert(QLatin1String(lineNumberCapKeyC), m_customParserSettings.lineNumberCap);
-    data.insert(QLatin1String(messageCapKeyC), m_customParserSettings.messageCap);
+    data.insert(QLatin1String(errorPatternKeyC), m_customParserSettings.error.pattern());
+    data.insert(QLatin1String(errorFileNameCapKeyC), m_customParserSettings.error.fileNameCap());
+    data.insert(QLatin1String(errorLineNumberCapKeyC), m_customParserSettings.error.lineNumberCap());
+    data.insert(QLatin1String(errorMessageCapKeyC), m_customParserSettings.error.messageCap());
+    data.insert(QLatin1String(errorChannelKeyC), m_customParserSettings.error.channel());
+    data.insert(QLatin1String(errorExampleKeyC), m_customParserSettings.error.example());
+    data.insert(QLatin1String(warningPatternKeyC), m_customParserSettings.warning.pattern());
+    data.insert(QLatin1String(warningFileNameCapKeyC), m_customParserSettings.warning.fileNameCap());
+    data.insert(QLatin1String(warningLineNumberCapKeyC), m_customParserSettings.warning.lineNumberCap());
+    data.insert(QLatin1String(warningMessageCapKeyC), m_customParserSettings.warning.messageCap());
+    data.insert(QLatin1String(warningChannelKeyC), m_customParserSettings.warning.channel());
+    data.insert(QLatin1String(warningExampleKeyC), m_customParserSettings.warning.example());
 
     return data;
 }
@@ -321,10 +319,29 @@ bool CustomToolChain::fromMap(const QVariantMap &data)
     m_cxx11Flags = data.value(QLatin1String(cxx11FlagsKeyC)).toStringList();
     setMkspecs(data.value(QLatin1String(mkspecsKeyC)).toString());
     m_outputParser = (OutputParser)data.value(QLatin1String(outputParserKeyC)).toInt();
-    m_customParserSettings.errorPattern = data.value(QLatin1String(errorPatternKeyC)).toString();
-    m_customParserSettings.fileNameCap = data.value(QLatin1String(fileNameCapKeyC)).toInt();
-    m_customParserSettings.lineNumberCap = data.value(QLatin1String(lineNumberCapKeyC)).toInt();
-    m_customParserSettings.messageCap = data.value(QLatin1String(messageCapKeyC)).toInt();
+    m_customParserSettings.error.setPattern(data.value(QLatin1String(errorPatternKeyC)).toString());
+    m_customParserSettings.error.setFileNameCap(data.value(QLatin1String(errorFileNameCapKeyC)).toInt());
+    m_customParserSettings.error.setLineNumberCap(data.value(QLatin1String(errorLineNumberCapKeyC)).toInt());
+    m_customParserSettings.error.setMessageCap(data.value(QLatin1String(errorMessageCapKeyC)).toInt());
+    m_customParserSettings.error.setChannel(
+                static_cast<CustomParserExpression::CustomParserChannel>(data.value(QLatin1String(errorChannelKeyC)).toInt()));
+    m_customParserSettings.error.setExample(data.value(QLatin1String(errorExampleKeyC)).toString());
+    m_customParserSettings.warning.setPattern(data.value(QLatin1String(warningPatternKeyC)).toString());
+    m_customParserSettings.warning.setFileNameCap(data.value(QLatin1String(warningFileNameCapKeyC)).toInt());
+    m_customParserSettings.warning.setLineNumberCap(data.value(QLatin1String(warningLineNumberCapKeyC)).toInt());
+    m_customParserSettings.warning.setMessageCap(data.value(QLatin1String(warningMessageCapKeyC)).toInt());
+    m_customParserSettings.warning.setChannel(
+                static_cast<CustomParserExpression::CustomParserChannel>(data.value(QLatin1String(warningChannelKeyC)).toInt()));
+    m_customParserSettings.warning.setExample(data.value(QLatin1String(warningExampleKeyC)).toString());
+
+    // TODO Creator 4.1: remove (added in 3.7 for compatibility)
+    if (m_customParserSettings.error.fileNameCap() == 0)
+        m_customParserSettings.error.setFileNameCap(data.value(QLatin1String(fileNameCapKeyC)).toInt());
+    if (m_customParserSettings.error.lineNumberCap() == 0)
+        m_customParserSettings.error.setLineNumberCap(data.value(QLatin1String(lineNumberCapKeyC)).toInt());
+    if (m_customParserSettings.error.messageCap() == 0)
+        m_customParserSettings.error.setMessageCap(data.value(QLatin1String(messageCapKeyC)).toInt());
+
     QTC_ASSERT(m_outputParser >= Gcc && m_outputParser < OutputParserCount, return false);
 
     return true;
@@ -390,7 +407,6 @@ namespace Internal {
 
 CustomToolChainFactory::CustomToolChainFactory()
 {
-    setId(Constants::CUSTOM_TOOLCHAIN_ID);
     setDisplayName(tr("Custom"));
 }
 
@@ -401,14 +417,13 @@ bool CustomToolChainFactory::canCreate()
 
 ToolChain *CustomToolChainFactory::create()
 {
-    return createToolChain(false);
+    return new CustomToolChain(ToolChain::ManualDetection);
 }
 
 // Used by the ToolChainManager to restore user-generated tool chains
 bool CustomToolChainFactory::canRestore(const QVariantMap &data)
 {
-    const QString id = idFromMap(data);
-    return id.startsWith(QLatin1String(Constants::CUSTOM_TOOLCHAIN_ID) + QLatin1Char(':'));
+    return typeIdFromMap(data) == Constants::CUSTOM_TOOLCHAIN_TYPEID;
 }
 
 ToolChain *CustomToolChainFactory::restore(const QVariantMap &data)
@@ -421,11 +436,6 @@ ToolChain *CustomToolChainFactory::restore(const QVariantMap &data)
     return 0;
 }
 
-CustomToolChain *CustomToolChainFactory::createToolChain(bool autoDetect)
-{
-    return new CustomToolChain(autoDetect ? ToolChain::AutoDetection : ToolChain::ManualDetection);
-}
-
 // --------------------------------------------------------------------------
 // Helper for ConfigWidget
 // --------------------------------------------------------------------------
@@ -518,17 +528,19 @@ CustomToolChainConfigWidget::CustomToolChainConfigWidget(CustomToolChain *tc) :
     m_predefinedDetails->updateSummaryText();
     m_headerDetails->updateSummaryText();
 
-    connect(m_compilerCommand, SIGNAL(changed(QString)), this, SIGNAL(dirty()));
-    connect(m_makeCommand, SIGNAL(changed(QString)), this, SIGNAL(dirty()));
-    connect(m_abiWidget, SIGNAL(abiChanged()), this, SIGNAL(dirty()));
-    connect(m_predefinedMacros, SIGNAL(textChanged()), this, SLOT(updateSummaries()));
-    connect(m_headerPaths, SIGNAL(textChanged()), this, SLOT(updateSummaries()));
-    connect(m_cxx11Flags, SIGNAL(textChanged(QString)), this, SIGNAL(dirty()));
-    connect(m_mkspecs, SIGNAL(textChanged(QString)), this, SIGNAL(dirty()));
-    connect(m_errorParserComboBox, SIGNAL(currentIndexChanged(int)),
-            this, SLOT(errorParserChanged(int)));
-    connect(m_customParserSettingsButton, SIGNAL(clicked()),
-            this, SLOT(openCustomParserSettingsDialog()));
+    connect(m_compilerCommand, &PathChooser::rawPathChanged, this, &ToolChainConfigWidget::dirty);
+    connect(m_makeCommand, &PathChooser::rawPathChanged, this, &ToolChainConfigWidget::dirty);
+    connect(m_abiWidget, &AbiWidget::abiChanged, this, &ToolChainConfigWidget::dirty);
+    connect(m_predefinedMacros, &QPlainTextEdit::textChanged,
+            this, &CustomToolChainConfigWidget::updateSummaries);
+    connect(m_headerPaths, &QPlainTextEdit::textChanged,
+            this, &CustomToolChainConfigWidget::updateSummaries);
+    connect(m_cxx11Flags, &QLineEdit::textChanged, this, &ToolChainConfigWidget::dirty);
+    connect(m_mkspecs, &QLineEdit::textChanged, this, &ToolChainConfigWidget::dirty);
+    connect(m_errorParserComboBox, static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
+            this, &CustomToolChainConfigWidget::errorParserChanged);
+    connect(m_customParserSettingsButton, &QAbstractButton::clicked,
+            this, &CustomToolChainConfigWidget::openCustomParserSettingsDialog);
     errorParserChanged(m_errorParserComboBox->currentIndex());
 }
 
@@ -585,7 +597,7 @@ void CustomToolChainConfigWidget::setFromToolchain()
     bool blocked = blockSignals(true);
     CustomToolChain *tc = static_cast<CustomToolChain *>(toolChain());
     m_compilerCommand->setFileName(tc->compilerCommand());
-    m_makeCommand->setFileName(FileName::fromString(tc->makeCommand(Utils::Environment())));
+    m_makeCommand->setFileName(FileName::fromString(tc->makeCommand(Environment())));
     m_abiWidget->setAbis(QList<Abi>(), tc->targetAbi());
     m_predefinedMacros->setPlainText(tc->rawPredefinedMacros().join(QLatin1Char('\n')));
     m_headerPaths->setPlainText(tc->headerPathsList().join(QLatin1Char('\n')));
@@ -601,7 +613,7 @@ bool CustomToolChainConfigWidget::isDirtyImpl() const
     CustomToolChain *tc = static_cast<CustomToolChain *>(toolChain());
     Q_ASSERT(tc);
     return m_compilerCommand->fileName() != tc->compilerCommand()
-            || m_makeCommand->path() != tc->makeCommand(Utils::Environment())
+            || m_makeCommand->path() != tc->makeCommand(Environment())
             || m_abiWidget->currentAbi() != tc->targetAbi()
             || m_predefinedDetails->entries() != tc->rawPredefinedMacros()
             || m_headerDetails->entries() != tc->headerPathsList()
