@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -218,7 +213,7 @@ QWidget *CustomWizardFieldPage::registerComboBox(const QString &fieldName,
     } while (false);
     registerField(fieldName, combo, "text", SIGNAL(text4Changed(QString)));
     // Connect to completeChanged() for derived classes that reimplement isComplete()
-    connect(combo, SIGNAL(text4Changed(QString)), SIGNAL(completeChanged()));
+    connect(combo, &TextFieldComboBox::text4Changed, this, &QWizardPage::completeChanged);
     return combo;
 } // QComboBox
 
@@ -232,7 +227,7 @@ QWidget *CustomWizardFieldPage::registerTextEdit(const QString &fieldName,
     textEdit->setAcceptRichText(acceptRichText);
     // Connect to completeChanged() for derived classes that reimplement isComplete()
     registerField(fieldName, textEdit, "plainText", SIGNAL(textChanged()));
-    connect(textEdit, SIGNAL(textChanged()), SIGNAL(completeChanged()));
+    connect(textEdit, &QTextEdit::textChanged, this, &QWizardPage::completeChanged);
     const QString defaultText = field.controlAttributes.value(QLatin1String("defaulttext"));
     m_textEdits.push_back(TextEditData(textEdit, defaultText));
     return textEdit;
@@ -255,11 +250,11 @@ QWidget *CustomWizardFieldPage::registerPathChooser(const QString &fieldName,
         pathChooser->setExpectedKind(PathChooser::Command);
     else if (expectedKind == QLatin1String("any"))
         pathChooser->setExpectedKind(PathChooser::Any);
-    pathChooser->setHistoryCompleter(QString::fromLatin1("PE.Custom.") + m_parameters->id + QLatin1Char('.') + field.name);
+    pathChooser->setHistoryCompleter(QString::fromLatin1("PE.Custom.") + m_parameters->id.toString() + QLatin1Char('.') + field.name);
 
-    registerField(fieldName, pathChooser, "path", SIGNAL(changed(QString)));
+    registerField(fieldName, pathChooser, "path", SIGNAL(rawPathChanged(QString)));
     // Connect to completeChanged() for derived classes that reimplement isComplete()
-    connect(pathChooser, SIGNAL(changed(QString)), SIGNAL(completeChanged()));
+    connect(pathChooser, &PathChooser::rawPathChanged, this, &QWizardPage::completeChanged);
     const QString defaultText = field.controlAttributes.value(QLatin1String("defaulttext"));
     m_pathChoosers.push_back(PathChooserData(pathChooser, defaultText));
     return pathChooser;
@@ -282,7 +277,7 @@ QWidget *CustomWizardFieldPage::registerCheckBox(const QString &fieldName,
         checkBox->setFalseText(falseTextIt.value());
     registerField(fieldName, checkBox, "text", SIGNAL(textChanged(QString)));
     // Connect to completeChanged() for derived classes that reimplement isComplete()
-    connect(checkBox, SIGNAL(textChanged(QString)), SIGNAL(completeChanged()));
+    connect(checkBox, &TextFieldCheckBox::textChanged, this, &QWizardPage::completeChanged);
     return checkBox;
 }
 
@@ -301,7 +296,7 @@ QWidget *CustomWizardFieldPage::registerLineEdit(const QString &fieldName,
     }
     registerField(fieldName, lineEdit, "text", SIGNAL(textEdited(QString)));
     // Connect to completeChanged() for derived classes that reimplement isComplete()
-    connect(lineEdit, SIGNAL(textEdited(QString)), SIGNAL(completeChanged()));
+    connect(lineEdit, &QLineEdit::textEdited, this, &QWizardPage::completeChanged);
 
     const QString defaultText = field.controlAttributes.value(QLatin1String("defaulttext"));
     const QString placeholderText = field.controlAttributes.value(QLatin1String("placeholdertext"));
@@ -439,7 +434,7 @@ CustomWizardPage::CustomWizardPage(const QSharedPointer<CustomWizardContext> &ct
 {
     m_pathChooser->setHistoryCompleter(QLatin1String("PE.ProjectDir.History"));
     addRow(tr("Path:"), m_pathChooser);
-    connect(m_pathChooser, SIGNAL(validChanged()), this, SIGNAL(completeChanged()));
+    connect(m_pathChooser, &PathChooser::validChanged, this, &QWizardPage::completeChanged);
 }
 
 QString CustomWizardPage::path() const
