@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -258,7 +253,7 @@ static QList<ProjectExplorer::ProjectAction> supportedNodeActions(ProjectExplore
     if (managesFiles)
         actions << ProjectExplorer::AddNewFile << ProjectExplorer::AddExistingFile;
     if (node->nodeType() == ProjectExplorer::FileNodeType
-            && !project->qbsProject().buildSystemFiles().contains(node->path().toString())) {
+            && !project->qbsProject().buildSystemFiles().contains(node->filePath().toString())) {
         actions << ProjectExplorer::RemoveFile << ProjectExplorer::Rename;
     }
     return actions;
@@ -420,7 +415,7 @@ void QbsGroupNode::updateQbsGroupData(const qbs::GroupData &grp, const QString &
     m_productPath = productPath;
     m_qbsGroupData = grp;
 
-    setPath(Utils::FileName::fromString(grp.location().filePath()));
+    setAbsoluteFilePathAndLine(Utils::FileName::fromString(grp.location().filePath()), line());
     setDisplayName(grp.name());
 
     QbsFileNode *idx = 0;
@@ -430,7 +425,7 @@ void QbsGroupNode::updateQbsGroupData(const qbs::GroupData &grp, const QString &
             break;
     }
     QTC_ASSERT(idx, return);
-    idx->setPathAndLine(Utils::FileName::fromString(grp.location().filePath()),
+    idx->setAbsoluteFilePathAndLine(Utils::FileName::fromString(grp.location().filePath()),
                         grp.location().line());
 
     setupFiles(this, grp, grp.allFilePaths(), productPath, updateExisting);
@@ -458,11 +453,17 @@ void QbsGroupNode::setupFiles(ProjectExplorer::FolderNode *root, const qbs::Grou
     FileTreeNode::reorder(&tree, productPath);
     FileTreeNode::simplify(&tree);
 
-    setupFolder(root, group, &tree, productPath, updateExisting);
+    QHash<QString, ProjectExplorer::FileType> fileTypeHash;
+    foreach (const qbs::SourceArtifact &sa, group.allSourceArtifacts())
+        fileTypeHash[sa.filePath()] = fileType(sa);
+
+    setupFolder(root, fileTypeHash, &tree, productPath, updateExisting);
 }
 
-void QbsGroupNode::setupFolder(ProjectExplorer::FolderNode *root, const qbs::GroupData &group,
-                               const FileTreeNode *fileTree, const QString &baseDir,
+void QbsGroupNode::setupFolder(ProjectExplorer::FolderNode *root,
+                               const QHash<QString, ProjectExplorer::FileType> &fileTypeHash,
+                               const FileTreeNode *fileTree,
+                               const QString &baseDir,
                                bool updateExisting)
 {
     // We only need to care about FileNodes and FolderNodes here. Everything else is
@@ -486,7 +487,8 @@ void QbsGroupNode::setupFolder(ProjectExplorer::FolderNode *root, const qbs::Gro
 
     foreach (FileTreeNode *c, fileTree->children) {
         Utils::FileName path = Utils::FileName::fromString(c->path());
-        const ProjectExplorer::FileType newFileType = fileType(group, *c);
+        const ProjectExplorer::FileType newFileType =
+                fileTypeHash.value(c->path(), ProjectExplorer::UnknownFileType);
         const bool isQrcFile = newFileType == ProjectExplorer::ResourceType;
 
         // Handle files:
@@ -494,7 +496,7 @@ void QbsGroupNode::setupFolder(ProjectExplorer::FolderNode *root, const qbs::Gro
             ProjectExplorer::FileNode *fn = 0;
             foreach (ProjectExplorer::FileNode *f, root->fileNodes()) {
                 // There can be one match only here!
-                if (f->path() != path || f->fileType() != newFileType)
+                if (f->filePath() != path || f->fileType() != newFileType)
                     continue;
                 fn = f;
                 break;
@@ -512,7 +514,7 @@ void QbsGroupNode::setupFolder(ProjectExplorer::FolderNode *root, const qbs::Gro
             ProjectExplorer::FolderNode *fn = 0;
             foreach (ProjectExplorer::FolderNode *f, root->subFolderNodes()) {
                 // There can be one match only here!
-                if (f->path() != path)
+                if (f->filePath() != path)
                     continue;
                 fn = f;
                 break;
@@ -537,7 +539,7 @@ void QbsGroupNode::setupFolder(ProjectExplorer::FolderNode *root, const qbs::Gro
             if (isQrcFile)
                 static_cast<ResourceTopLevelNode *>(fn)->update();
             else
-                setupFolder(fn, group, c, c->path(), updateExisting);
+                setupFolder(fn, fileTypeHash, c, c->path(), updateExisting);
 
         }
     }
@@ -546,17 +548,9 @@ void QbsGroupNode::setupFolder(ProjectExplorer::FolderNode *root, const qbs::Gro
     root->addFileNodes(filesToAdd);
 }
 
-ProjectExplorer::FileType QbsGroupNode::fileType(const qbs::GroupData &group,
-                                                 const FileTreeNode &fileNode)
+ProjectExplorer::FileType QbsGroupNode::fileType(const qbs::SourceArtifact &artifact)
 {
-    if (!group.isValid())
-        return ProjectExplorer::UnknownFileType;
-    const qbs::SourceArtifact artifact = Utils::findOrDefault(group.allSourceArtifacts(),
-            [&fileNode](const qbs::SourceArtifact &sa) { return sa.filePath() == fileNode.path(); });
-    QTC_ASSERT(artifact.isValid() || !fileNode.isFile(),
-               qDebug() << fileNode.path() << group.name(); return ProjectExplorer::UnknownFileType);
-    if (!artifact.isValid())
-        return ProjectExplorer::UnknownFileType;
+    QTC_ASSERT(artifact.isValid(), return ProjectExplorer::UnknownFileType);
 
     if (artifact.fileTags().contains(QLatin1String("c"))
             || artifact.fileTags().contains(QLatin1String("cpp"))
@@ -570,6 +564,8 @@ ProjectExplorer::FileType QbsGroupNode::fileType(const qbs::GroupData &group,
         return ProjectExplorer::ResourceType;
     if (artifact.fileTags().contains(QLatin1String("ui")))
         return ProjectExplorer::FormType;
+    if (artifact.fileTags().contains(QLatin1String("scxml")))
+        return ProjectExplorer::StateChartType;
     return ProjectExplorer::UnknownFileType;
 }
 
@@ -671,7 +667,7 @@ void QbsProductNode::setQbsProductData(const qbs::Project &project, const qbs::P
     bool updateExisting = productWasEnabled != productIsEnabled;
 
     setDisplayName(QbsProject::productDisplayName(project, prd));
-    setPath(Utils::FileName::fromString(prd.location().filePath()));
+    setAbsoluteFilePathAndLine(Utils::FileName::fromString(prd.location().filePath()), line());
     const QString &productPath = QFileInfo(prd.location().filePath()).absolutePath();
 
     // Find the QbsFileNode we added earlier:
@@ -682,7 +678,7 @@ void QbsProductNode::setQbsProductData(const qbs::Project &project, const qbs::P
             break;
     }
     QTC_ASSERT(idx, return);
-    idx->setPathAndLine(Utils::FileName::fromString(prd.location().filePath()),
+    idx->setAbsoluteFilePathAndLine(Utils::FileName::fromString(prd.location().filePath()),
                         prd.location().line());
 
     QList<ProjectExplorer::ProjectNode *> toAdd;
@@ -813,11 +809,11 @@ bool QbsProjectNode::showInSimpleTree() const
 void QbsProjectNode::ctor()
 {
     if (m_projectIcon.isNull())
-        m_projectIcon = generateIcon(QString::fromLatin1(QtSupport::Constants::ICON_QT_PROJECT));
+        m_projectIcon = generateIcon(QString::fromLatin1(ProjectExplorer::Constants::FILEOVERLAY_QT));
 
     setIcon(m_projectIcon);
     addFileNodes(QList<ProjectExplorer::FileNode *>()
-                 << new ProjectExplorer::FileNode(path(), ProjectExplorer::ProjectFileType, false));
+                 << new ProjectExplorer::FileNode(filePath(), ProjectExplorer::ProjectFileType, false));
 }
 
 QbsProductNode *QbsProjectNode::findProductNode(const QString &uniqueName)
