@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,21 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -41,11 +37,12 @@
 #include <projectexplorer/toolchain.h>
 #include <projectexplorer/projectexplorer.h>
 #include <projectexplorer/headerpath.h>
-#include <qtsupport/debugginghelperbuildtask.h>
+#include <qtsupport/qtkitinformation.h>
 #include <qtsupport/qtsupportconstants.h>
 
 #include <utils/algorithm.h>
 #include <utils/hostosinfo.h>
+#include <utils/macroexpander.h>
 #include <utils/qtcassert.h>
 #include <utils/runextensions.h>
 #include <utils/synchronousprocess.h>
@@ -58,6 +55,7 @@
 #include <QFuture>
 #include <QCoreApplication>
 #include <QProcess>
+#include <QRegExp>
 
 using namespace Core;
 using namespace QtSupport;
@@ -72,6 +70,30 @@ static const char QTVERSIONQMAKEPATH[] = "QMakePath";
 static const char MKSPEC_VALUE_LIBINFIX[] = "QT_LIBINFIX";
 static const char MKSPEC_VALUE_NAMESPACE[] = "QT_NAMESPACE";
 
+static QSet<Id> versionedIds(const QByteArray &prefix, int major, int minor)
+{
+    QSet<Id> result;
+    result.insert(Id::fromName(prefix));
+
+    if (major < 0)
+        return result;
+
+    const QByteArray majorStr = QString::number(major).toLatin1();
+    const QByteArray featureMajor = prefix + majorStr;
+    const QByteArray featureDotMajor = prefix + '.' + majorStr;
+
+    result.insert(Id::fromName(featureMajor));
+    result.insert(Id::fromName(featureDotMajor));
+
+    for (int i = 0; i <= minor; ++i) {
+        const QByteArray minorStr = QString::number(i).toLatin1();
+        result.insert(Id::fromName(featureMajor + '.' + minorStr));
+        result.insert(Id::fromName(featureDotMajor + '.' + minorStr));
+    }
+
+    return result;
+}
+
 ///////////////
 // QtVersionNumber
 ///////////////
@@ -92,6 +114,28 @@ QtVersionNumber::QtVersionNumber()
     majorVersion = minorVersion = patchVersion = -1;
 }
 
+QSet<Id> QtVersionNumber::features() const
+{
+    return versionedIds(Constants::FEATURE_QT_PREFIX, majorVersion, minorVersion);
+}
+
+bool QtVersionNumber::matches(int major, int minor, int patch) const
+{
+    if (major < 0)
+        return true;
+    if (major != majorVersion)
+        return false;
+
+    if (minor < 0)
+        return true;
+    if (minor != minorVersion)
+        return false;
+
+    if (patch < 0)
+        return true;
+    return (patch == patchVersion);
+}
+
 bool QtVersionNumber::operator <(const QtVersionNumber &b) const
 {
     if (majorVersion != b.majorVersion)
@@ -157,6 +201,44 @@ BaseQtVersion::BaseQtVersion(const FileName &qmakeCommand, bool isAutodetected,
     ctor(qmakeCommand);
 }
 
+BaseQtVersion::BaseQtVersion(const BaseQtVersion &other) :
+    m_id(other.m_id),
+    m_isAutodetected(other.m_isAutodetected),
+    m_hasQmlDump(other.m_hasQmlDump),
+    m_mkspecUpToDate(other.m_mkspecUpToDate),
+    m_mkspecReadUpToDate(other.m_mkspecReadUpToDate),
+    m_defaultConfigIsDebug(other.m_defaultConfigIsDebug),
+    m_defaultConfigIsDebugAndRelease(other.m_defaultConfigIsDebugAndRelease),
+    m_frameworkBuild(other.m_frameworkBuild),
+    m_versionInfoUpToDate(other.m_versionInfoUpToDate),
+    m_installed(other.m_installed),
+    m_hasExamples(other.m_hasExamples),
+    m_hasDemos(other.m_hasDemos),
+    m_hasDocumentation(other.m_hasDocumentation),
+    m_qmakeIsExecutable(other.m_qmakeIsExecutable),
+    m_hasQtAbis(other.m_hasQtAbis),
+    m_configValues(other.m_configValues),
+    m_qtConfigValues(other.m_qtConfigValues),
+    m_unexpandedDisplayName(other.m_unexpandedDisplayName),
+    m_autodetectionSource(other.m_autodetectionSource),
+    m_sourcePath(other.m_sourcePath),
+    m_mkspec(other.m_mkspec),
+    m_mkspecFullPath(other.m_mkspecFullPath),
+    m_mkspecValues(other.m_mkspecValues),
+    m_versionInfo(other.m_versionInfo),
+    m_qmakeCommand(other.m_qmakeCommand),
+    m_qtVersionString(other.m_qtVersionString),
+    m_uicCommand(other.m_uicCommand),
+    m_designerCommand(other.m_designerCommand),
+    m_linguistCommand(other.m_linguistCommand),
+    m_qmlsceneCommand(other.m_qmlsceneCommand),
+    m_qmlviewerCommand(other.m_qmlviewerCommand),
+    m_qscxmlcCommand(other.m_qscxmlcCommand),
+    m_qtAbis(other.m_qtAbis)
+{
+    setupExpander();
+}
+
 BaseQtVersion::BaseQtVersion()
     :  m_id(-1), m_isAutodetected(false),
     m_hasQmlDump(false),
@@ -183,20 +265,100 @@ void BaseQtVersion::ctor(const FileName &qmakePath)
     m_linguistCommand.clear();
     m_qmlviewerCommand.clear();
     m_uicCommand.clear();
+    m_qscxmlcCommand.clear();
     m_mkspecUpToDate = false;
     m_mkspecReadUpToDate = false;
     m_versionInfoUpToDate = false;
     m_hasQtAbis = false;
     m_qtVersionString.clear();
     m_sourcePath.clear();
+    setupExpander();
+}
+
+void BaseQtVersion::setupExpander()
+{
+    m_expander.setDisplayName(
+        QtKitInformation::tr("Qt version"));
+
+    m_expander.registerVariable("Qt:Version",
+        QtKitInformation::tr("The version string of the current Qt version."),
+        [this] { return qtVersionString(); });
+
+    m_expander.registerVariable("Qt:Type",
+        QtKitInformation::tr("The type of the current Qt version."),
+        [this] { return type(); });
+
+    m_expander.registerVariable("Qt:Mkspec",
+        QtKitInformation::tr("The mkspec of the current Qt version."),
+        [this] { return mkspec().toUserOutput(); });
+
+    m_expander.registerVariable("Qt:QT_INSTALL_PREFIX",
+        QtKitInformation::tr("The installation prefix of the current Qt version."),
+        [this] { return qmakeProperty(m_versionInfo, "QT_INSTALL_PREFIX"); });
+
+    m_expander.registerVariable("Qt:QT_INSTALL_DATA",
+        QtKitInformation::tr("The installation location of the current Qt version's data."),
+        [this] { return qmakeProperty(m_versionInfo, "QT_INSTALL_DATA"); });
+
+    m_expander.registerVariable("Qt:QT_INSTALL_HEADERS",
+        QtKitInformation::tr("The installation location of the current Qt version's header files."),
+        [this] { return qmakeProperty(m_versionInfo, "QT_INSTALL_HEADERS"); });
+
+    m_expander.registerVariable("Qt:QT_INSTALL_LIBS",
+        QtKitInformation::tr("The installation location of the current Qt version's library files."),
+        [this] { return qmakeProperty(m_versionInfo, "QT_INSTALL_LIBS"); });
+
+    m_expander.registerVariable("Qt:QT_INSTALL_DOCS",
+        QtKitInformation::tr("The installation location of the current Qt version's documentation files."),
+        [this] { return qmakeProperty(m_versionInfo, "QT_INSTALL_DOCS"); });
+
+    m_expander.registerVariable("Qt:QT_INSTALL_BINS",
+        QtKitInformation::tr("The installation location of the current Qt version's executable files."),
+        [this] { return qmakeProperty(m_versionInfo, "QT_INSTALL_BINS"); });
+
+    m_expander.registerVariable("Qt:QT_INSTALL_PLUGINS",
+        QtKitInformation::tr("The installation location of the current Qt version's plugins."),
+        [this] { return qmakeProperty(m_versionInfo, "QT_INSTALL_PLUGINS"); });
+
+    m_expander.registerVariable("Qt:QT_INSTALL_IMPORTS",
+        QtKitInformation::tr("The installation location of the current Qt version's imports."),
+        [this] { return qmakeProperty(m_versionInfo, "QT_INSTALL_IMPORTS"); });
+
+    m_expander.registerVariable("Qt:QT_INSTALL_TRANSLATIONS",
+        QtKitInformation::tr("The installation location of the current Qt version's translation files."),
+        [this] { return qmakeProperty(m_versionInfo, "QT_INSTALL_TRANSLATIONS"); });
+
+    m_expander.registerVariable("Qt:QT_INSTALL_CONFIGURATION",
+        QtKitInformation::tr("The installation location of the current Qt version's translation files."),
+        [this] { return qmakeProperty(m_versionInfo, "QT_INSTALL_CONFIGURATION"); });
+
+    m_expander.registerVariable("Qt:QT_INSTALL_EXAMPLES",
+        QtKitInformation::tr("The installation location of the current Qt version's examples."),
+        [this] { return qmakeProperty(m_versionInfo, "QT_INSTALL_EXAMPLES"); });
+
+    m_expander.registerVariable("Qt:QT_INSTALL_DEMOS",
+        QtKitInformation::tr("The installation location of the current Qt version's demos."),
+        [this] { return qmakeProperty(m_versionInfo, "QT_INSTALL_DEMOS"); });
+
+    m_expander.registerVariable("Qt:QMAKE_MKSPECS",
+        QtKitInformation::tr("The current Qt version's default mkspecs."),
+        [this] { return qmakeProperty(m_versionInfo, "QMAKE_MKSPECS"); });
+
+    m_expander.registerVariable("Qt:QMAKE_VERSION",
+        QtKitInformation::tr("The current Qt's qmake version."),
+        [this] { return qmakeProperty(m_versionInfo, "QMAKE_VERSION"); });
+
+//    FIXME: Re-enable once we can detect expansion loops.
+//    m_expander.registerVariable("Qt:Name",
+//        QtKitInformation::tr("The display name of the current Qt version."),
+//        [this] { return displayName(); });
 }
 
 BaseQtVersion::~BaseQtVersion()
 {
 }
 
-QString BaseQtVersion::defaultDisplayName(const QString &versionString, const FileName &qmakePath,
-                                          bool fromPath)
+QString BaseQtVersion::defaultUnexpandedDisplayName(const FileName &qmakePath, bool fromPath)
 {
     QString location;
     if (qmakePath.isEmpty()) {
@@ -222,66 +384,87 @@ QString BaseQtVersion::defaultDisplayName(const QString &versionString, const Fi
     }
 
     return fromPath ?
-        QCoreApplication::translate("QtVersion", "Qt %1 in PATH (%2)").arg(versionString, location) :
-        QCoreApplication::translate("QtVersion", "Qt %1 (%2)").arg(versionString, location);
+        QCoreApplication::translate("QtVersion", "Qt %{Qt:Version} in PATH (%2)").arg(location) :
+        QCoreApplication::translate("QtVersion", "Qt %{Qt:Version} (%2)").arg(location);
 }
 
-FeatureSet BaseQtVersion::availableFeatures() const
+QSet<Id> BaseQtVersion::availableFeatures() const
 {
-    FeatureSet features = FeatureSet(Constants::FEATURE_QWIDGETS)
-            | FeatureSet(Constants::FEATURE_QT)
-            | FeatureSet(Constants::FEATURE_QT_WEBKIT)
-            | FeatureSet(Constants::FEATURE_QT_CONSOLE);
+    QSet<Id> features = qtVersion().features(); // Qt Version features
 
-     if (qtVersion() < QtVersionNumber(4, 7, 0))
-         return features;
+    features.insert(Constants::FEATURE_QWIDGETS);
+    features.insert(Constants::FEATURE_QT_WEBKIT);
+    features.insert(Constants::FEATURE_QT_CONSOLE);
 
-     features |= FeatureSet(Constants::FEATURE_QT_QUICK);
-     features |= FeatureSet(Constants::FEATURE_QT_QUICK_1);
+    if (qtVersion() < QtVersionNumber(4, 7, 0))
+        return features;
 
-     if (qtVersion() < QtVersionNumber(4, 7, 1))
-         return features;
+    features.unite(versionedIds(Constants::FEATURE_QT_QUICK_PREFIX, 1, 0));
 
-     features |= FeatureSet(Constants::FEATURE_QT_QUICK_1_1);
+    if (qtVersion().matches(4, 7, 0))
+        return features;
 
-     if (qtVersion() < QtVersionNumber(5, 0, 0))
-         return features;
+    features.unite(versionedIds(Constants::FEATURE_QT_QUICK_PREFIX, 1, 1));
 
-     features |= FeatureSet(Constants::FEATURE_QT_QUICK_2);
-     features |= FeatureSet(Constants::FEATURE_QT_QUICK_2_0);
+    if (qtVersion().matches(4))
+        return features;
 
-     if (qtVersion() < QtVersionNumber(5, 1, 0))
-         return features;
+    features.unite(versionedIds(Constants::FEATURE_QT_QUICK_PREFIX, 2, 0));
 
-     features |= FeatureSet(Constants::FEATURE_QT_QUICK_2_1);
-     features |= FeatureSet(Constants::FEATURE_QT_QUICK_CONTROLS);
-     features |= FeatureSet(Constants::FEATURE_QT_QUICK_CONTROLS_1);
-     features |= FeatureSet(Constants::FEATURE_QT_QUICK_CONTROLS_1_0);
+    if (qtVersion().matches(5, 0))
+        return features;
 
-     if (qtVersion() < QtVersionNumber(5, 2, 0))
-         return features;
+    features.unite(versionedIds(Constants::FEATURE_QT_QUICK_PREFIX, 2, 1));
+    features.unite(versionedIds(Constants::FEATURE_QT_QUICK_CONTROLS_PREFIX, 1, 0));
 
-     features |= FeatureSet(Constants::FEATURE_QT_QUICK_2_2);
-     features |= FeatureSet(Constants::FEATURE_QT_QUICK_CONTROLS_1_1);
+    if (qtVersion().matches(5, 1))
+        return features;
 
-     return features;
-}
+    features.unite(versionedIds(Constants::FEATURE_QT_QUICK_PREFIX, 2, 2));
+    features.unite(versionedIds(Constants::FEATURE_QT_QUICK_CONTROLS_PREFIX, 1, 1));
 
-QString BaseQtVersion::platformName() const
-{
-    return QString();
-}
+    if (qtVersion().matches(5, 2))
+        return features;
 
-QString BaseQtVersion::platformDisplayName() const
-{
-    return platformName();
-}
+    features.unite(versionedIds(Constants::FEATURE_QT_QUICK_PREFIX, 2, 3));
+    features.unite(versionedIds(Constants::FEATURE_QT_QUICK_CONTROLS_PREFIX, 1, 2));
 
-bool BaseQtVersion::supportsPlatform(const QString &platform) const
-{
-    if (platform.isEmpty()) // empty target == target independent
-        return true;
-    return platform == platformName();
+    if (qtVersion().matches(5, 3))
+        return features;
+
+    features.insert(Constants::FEATURE_QT_QUICK_UI_FILES);
+
+    features.unite(versionedIds(Constants::FEATURE_QT_QUICK_PREFIX, 2, 4));
+    features.unite(versionedIds(Constants::FEATURE_QT_QUICK_CONTROLS_PREFIX, 1, 3));
+
+    if (qtVersion().matches(5, 4))
+        return features;
+
+    features.insert(Constants::FEATURE_QT_3D);
+
+    features.unite(versionedIds(Constants::FEATURE_QT_QUICK_PREFIX, 2, 5));
+    features.unite(versionedIds(Constants::FEATURE_QT_QUICK_CONTROLS_PREFIX, 1, 4));
+    features.unite(versionedIds(Constants::FEATURE_QT_CANVAS3D_PREFIX, 1, 0));
+
+    if (qtVersion().matches(5, 5))
+        return features;
+
+    features.unite(versionedIds(Constants::FEATURE_QT_QUICK_PREFIX, 2, 6));
+    features.unite(versionedIds(Constants::FEATURE_QT_QUICK_CONTROLS_PREFIX, 1, 5));
+    features.unite(versionedIds(Constants::FEATURE_QT_LABS_CONTROLS_PREFIX, 1, 0));
+    features.unite(versionedIds(Constants::FEATURE_QT_CANVAS3D_PREFIX, 1, 1));
+
+    if (qtVersion().matches(5, 6))
+        return features;
+
+    features.unite(versionedIds(Constants::FEATURE_QT_QUICK_PREFIX, 2, 7));
+    features.unite(versionedIds(Constants::FEATURE_QT_QUICK_CONTROLS_2_PREFIX, 2, 0));
+    features.subtract(versionedIds(Constants::FEATURE_QT_LABS_CONTROLS_PREFIX, 1, 0));
+
+    if (qtVersion().matches(5, 7))
+        return features;
+
+    return features;
 }
 
 QList<Task> BaseQtVersion::validateKit(const Kit *k)
@@ -295,6 +478,15 @@ QList<Task> BaseQtVersion::validateKit(const Kit *k)
     if (qtAbis.isEmpty()) // No need to test if Qt does not know anyway...
         return result;
 
+    const Id dt = DeviceTypeKitInformation::deviceTypeId(k);
+    const QSet<Id> tdt = targetDeviceTypes();
+    if (!tdt.isEmpty() && !tdt.contains(dt)) {
+        result << Task(Task::Warning,
+                       QCoreApplication::translate("BaseQtVersion",
+                                                   "Device type is not supported by Qt version."),
+                       FileName(), -1, ProjectExplorer::Constants::TASK_CATEGORY_BUILDSYSTEM);
+    }
+
     ToolChain *tc = ToolChainKitInformation::toolChain(k);
     if (tc) {
         Abi targetAbi = tc->targetAbi();
@@ -403,7 +595,7 @@ void BaseQtVersion::fromMap(const QVariantMap &map)
     m_id = map.value(QLatin1String(Constants::QTVERSIONID)).toInt();
     if (m_id == -1) // this happens on adding from installer, see updateFromInstaller => get a new unique id
         m_id = QtVersionManager::getUniqueId();
-    m_displayName = map.value(QLatin1String(Constants::QTVERSIONNAME)).toString();
+    m_unexpandedDisplayName = map.value(QLatin1String(Constants::QTVERSIONNAME)).toString();
     m_isAutodetected = map.value(QLatin1String(QTVERSIONAUTODETECTED)).toBool();
     if (m_isAutodetected)
         m_autodetectionSource = map.value(QLatin1String(QTVERSIONAUTODETECTIONSOURCE)).toString();
@@ -426,7 +618,7 @@ QVariantMap BaseQtVersion::toMap() const
 {
     QVariantMap result;
     result.insert(QLatin1String(Constants::QTVERSIONID), uniqueId());
-    result.insert(QLatin1String(Constants::QTVERSIONNAME), displayName());
+    result.insert(QLatin1String(Constants::QTVERSIONNAME), unexpandedDisplayName());
     result.insert(QLatin1String(QTVERSIONAUTODETECTED), isAutodetected());
     if (isAutodetected())
         result.insert(QLatin1String(QTVERSIONAUTODETECTIONSOURCE), autodetectionSource());
@@ -478,19 +670,6 @@ QStringList BaseQtVersion::warningReason() const
     return ret;
 }
 
-ToolChain *BaseQtVersion::preferredToolChain(const FileName &ms) const
-{
-    const FileName spec = ms.isEmpty() ? mkspec() : ms;
-
-    QList<ToolChain *> toolchains = ToolChainManager::toolChains();
-    return Utils::findOr(toolchains,
-                         toolchains.isEmpty() ? 0 : toolchains.first(),
-                         [&spec, this](ToolChain *tc) {
-                                return qtAbis().contains(tc->targetAbi())
-                                         && tc->suggestedMkspecList().contains(spec);
-                          });
-}
-
 FileName BaseQtVersion::qmakeCommand() const
 {
     return m_qmakeCommand;
@@ -507,6 +686,8 @@ QList<Abi> BaseQtVersion::qtAbis() const
 
 bool BaseQtVersion::equals(BaseQtVersion *other)
 {
+    if (m_qmakeCommand != other->m_qmakeCommand)
+        return false;
     if (type() != other->type())
         return false;
     if (uniqueId() != other->uniqueId())
@@ -541,12 +722,17 @@ void BaseQtVersion::setAutoDetectionSource(const QString &autodetectionSource)
 
 QString BaseQtVersion::displayName() const
 {
-    return m_displayName;
+    return m_expander.expand(m_unexpandedDisplayName);
 }
 
-void BaseQtVersion::setDisplayName(const QString &name)
+QString BaseQtVersion::unexpandedDisplayName() const
 {
-    m_displayName = name;
+    return m_unexpandedDisplayName;
+}
+
+void BaseQtVersion::setUnexpandedDisplayName(const QString &name)
+{
+    m_unexpandedDisplayName = name;
 }
 
 QString BaseQtVersion::toHtml(bool verbose) const
@@ -681,6 +867,16 @@ QString BaseQtVersion::qmlviewerCommand() const
     return m_qmlviewerCommand;
 }
 
+QString BaseQtVersion::qscxmlcCommand() const
+{
+    if (!isValid())
+        return QString();
+
+    if (m_qscxmlcCommand.isNull())
+        m_qscxmlcCommand = findQtBinary(QScxmlc);
+    return m_qscxmlcCommand;
+}
+
 QString BaseQtVersion::findQtBinary(Binaries binary) const
 {
     QString baseDir;
@@ -700,6 +896,7 @@ QString BaseQtVersion::findQtBinary(Binaries binary) const
             baseDir = m_mkspecValues.value(QLatin1String("QT.designer.bins"));
             break;
         case Uic:
+        case QScxmlc:
             baseDir = qmakeProperty("QT_HOST_BINS");
             break;
         default:
@@ -734,6 +931,7 @@ QString BaseQtVersion::findQtBinary(Binaries binary) const
     case Linguist:
         if (HostOsInfo::isMacHost())
             possibleCommands << QLatin1String("Linguist.app/Contents/MacOS/Linguist");
+        else
             possibleCommands << HostOsInfo::withExecutableSuffix(QLatin1String("linguist"));
         break;
     case Uic:
@@ -744,6 +942,9 @@ QString BaseQtVersion::findQtBinary(Binaries binary) const
                              << QLatin1String("uic");
         }
         break;
+    case QScxmlc:
+        possibleCommands << HostOsInfo::withExecutableSuffix(QLatin1String("qscxmlc"));
+        break;
     default:
         Q_ASSERT(false);
     }
@@ -856,7 +1057,7 @@ FileName BaseQtVersion::mkspecFor(ToolChain *tc) const
     if (!tc)
         return versionSpec;
 
-    const QList<FileName> tcSpecList = tc->suggestedMkspecList();
+    const FileNameList tcSpecList = tc->suggestedMkspecList();
     if (tcSpecList.contains(versionSpec))
         return versionSpec;
     foreach (const FileName &tcSpec, tcSpecList) {
@@ -943,35 +1144,30 @@ void BaseQtVersion::updateVersionInfo() const
     // Now check for a qt that is configured with a prefix but not installed
     QString installDir = qmakeProperty(m_versionInfo, "QT_HOST_BINS");
     if (!installDir.isNull()) {
-        QFileInfo fi(installDir);
-        if (!fi.exists())
+        if (!QFileInfo::exists(installDir))
             m_installed = false;
     }
     // Framework builds for Qt 4.8 don't use QT_INSTALL_HEADERS
     // so we don't check on mac
     if (!HostOsInfo::isMacHost()) {
         if (!qtHeaderData.isNull()) {
-            const QFileInfo fi(qtHeaderData);
-            if (!fi.exists())
+            if (!QFileInfo::exists(qtHeaderData))
                 m_installed = false;
         }
     }
     const QString qtInstallDocs = qmakeProperty(m_versionInfo, "QT_INSTALL_DOCS");
     if (!qtInstallDocs.isNull()) {
-        const QFileInfo fi(qtInstallDocs);
-        if (fi.exists())
+        if (QFileInfo::exists(qtInstallDocs))
             m_hasDocumentation = true;
     }
     const QString qtInstallExamples = qmakeProperty(m_versionInfo, "QT_INSTALL_EXAMPLES");
     if (!qtInstallExamples.isNull()) {
-        const QFileInfo fi(qtInstallExamples);
-        if (fi.exists())
+        if (QFileInfo::exists(qtInstallExamples))
             m_hasExamples = true;
     }
     const QString qtInstallDemos = qmakeProperty(m_versionInfo, "QT_INSTALL_DEMOS");
     if (!qtInstallDemos.isNull()) {
-        const QFileInfo fi(qtInstallDemos);
-        if (fi.exists())
+        if (QFileInfo::exists(qtInstallDemos))
             m_hasDemos = true;
     }
     m_qtVersionString = qmakeProperty(m_versionInfo, "QT_VERSION");
@@ -1020,7 +1216,7 @@ bool BaseQtVersion::hasDemos() const
 
 QString BaseQtVersion::demosPath() const
 {
-    return qmakeProperty("QT_INSTALL_DEMOS");
+    return QFileInfo(qmakeProperty("QT_INSTALL_DEMOS")).canonicalFilePath();
 }
 
 QString BaseQtVersion::frameworkInstallPath() const
@@ -1038,7 +1234,7 @@ bool BaseQtVersion::hasExamples() const
 
 QString BaseQtVersion::examplesPath() const
 {
-    return qmakeProperty("QT_INSTALL_EXAMPLES");
+    return QFileInfo(qmakeProperty("QT_INSTALL_EXAMPLES")).canonicalFilePath();
 }
 
 QStringList BaseQtVersion::configValues() const
@@ -1053,12 +1249,9 @@ QStringList BaseQtVersion::qtConfigValues() const
     return m_qtConfigValues;
 }
 
-QList<HeaderPath> BaseQtVersion::systemHeaderPathes(const Kit *k) const
+MacroExpander *BaseQtVersion::macroExpander() const
 {
-    Q_UNUSED(k);
-    QList<HeaderPath> result;
-    result.append(HeaderPath(mkspecPath().toString(), HeaderPath::GlobalHeaderPath));
-    return result;
+    return &m_expander;
 }
 
 void BaseQtVersion::addToEnvironment(const Kit *k, Environment &env) const
@@ -1128,11 +1321,6 @@ void BaseQtVersion::recheckDumper()
     m_versionInfoUpToDate = false;
 }
 
-bool BaseQtVersion::supportsShadowBuilds() const
-{
-    return true;
-}
-
 QList<Task> BaseQtVersion::reportIssuesImpl(const QString &proFile, const QString &buildDir) const
 {
     QList<Task> results;
@@ -1326,9 +1514,16 @@ FileName BaseQtVersion::mkspecFromVersionInfo(const QHash<QString, QString> &ver
                         const QList<QByteArray> &temp = line.split('=');
                         if (temp.size() == 2) {
                             QString possibleFullPath = QString::fromLocal8Bit(temp.at(1).trimmed().constData());
+                            if (possibleFullPath.contains(QLatin1Char('$'))) { // QTBUG-28792
+                                const QRegExp rex(QLatin1String("\\binclude\\(([^)]+)/qmake\\.conf\\)"));
+                                if (rex.indexIn(QString::fromLocal8Bit(f2.readAll())) != -1) {
+                                    possibleFullPath = mkspecFullPath.toString() + QLatin1Char('/')
+                                            + rex.cap(1);
+                                }
+                            }
                             // We sometimes get a mix of different slash styles here...
                             possibleFullPath = possibleFullPath.replace(QLatin1Char('\\'), QLatin1Char('/'));
-                            if (QFileInfo(possibleFullPath).exists()) // Only if the path exists
+                            if (QFileInfo::exists(possibleFullPath)) // Only if the path exists
                                 mkspecFullPath = FileName::fromUserInput(possibleFullPath);
                         }
                         break;
@@ -1372,6 +1567,10 @@ FileName BaseQtVersion::mkspecFromVersionInfo(const QHash<QString, QString> &ver
 
 FileName BaseQtVersion::sourcePath(const QHash<QString, QString> &versionInfo)
 {
+    const QString qt5Source = qmakeProperty(versionInfo, "QT_INSTALL_PREFIX/src");
+    if (!qt5Source.isEmpty())
+        return Utils::FileName::fromString(QFileInfo(qt5Source).canonicalFilePath());
+
     const QString installData = qmakeProperty(versionInfo, "QT_INSTALL_PREFIX");
     QString sourcePath = installData;
     QFile qmakeCache(installData + QLatin1String("/.qmake.cache"));
@@ -1390,7 +1589,41 @@ FileName BaseQtVersion::sourcePath(const QHash<QString, QString> &versionInfo)
             }
         }
     }
-    return FileName::fromUserInput(sourcePath);
+    return FileName::fromUserInput(QFileInfo(sourcePath).canonicalFilePath());
+}
+
+bool BaseQtVersion::isInSourceDirectory(const Utils::FileName &filePath)
+{
+    const Utils::FileName &source = sourcePath();
+    if (source.isEmpty())
+        return false;
+    QDir dir = QDir(source.toString());
+    if (dir.dirName() == QLatin1String("qtbase"))
+        dir.cdUp();
+    return filePath.isChildOf(dir);
+}
+
+bool BaseQtVersion::isSubProject(const Utils::FileName &filePath)
+{
+    const Utils::FileName &source = sourcePath();
+    if (!source.isEmpty()) {
+        QDir dir = QDir(source.toString());
+        if (dir.dirName() == QLatin1String("qtbase"))
+            dir.cdUp();
+
+        if (filePath.isChildOf(dir))
+            return true;
+    }
+
+    const QString &examples = examplesPath();
+    if (!examples.isEmpty() && filePath.isChildOf(QDir(examples)))
+        return true;
+
+    const QString &demos = demosPath();
+    if (!demos.isEmpty() && filePath.isChildOf(QDir(demos)))
+        return true;
+
+    return false;
 }
 
 bool BaseQtVersion::isQmlDebuggingSupported(Kit *k, QString *reason)
@@ -1413,47 +1646,60 @@ bool BaseQtVersion::isQmlDebuggingSupported(QString *reason) const
         return false;
     }
 
-    if (qtVersion() < QtVersionNumber(4, 8, 0)) {
+    if (qtVersion() < QtVersionNumber(5, 0, 0)) {
         if (reason)
-            *reason = QCoreApplication::translate("BaseQtVersion", "Requires Qt 4.8.0 or newer.");
+            *reason = QCoreApplication::translate("BaseQtVersion", "Requires Qt 5.0.0 or newer.");
         return false;
     }
 
     return true;
 }
 
-void BaseQtVersion::buildDebuggingHelper(Kit *k, int tools)
+bool BaseQtVersion::isQtQuickCompilerSupported(Kit *k, QString *reason)
 {
+    QTC_ASSERT(k, return false);
     BaseQtVersion *version = QtKitInformation::qtVersion(k);
-    ToolChain *tc = ToolChainKitInformation::toolChain(k);
-    if (!k || !version || !tc)
-        return;
-
-    version->buildDebuggingHelper(tc, tools);
+    if (!version) {
+        if (reason)
+            *reason = QCoreApplication::translate("BaseQtVersion", "No Qt version.");
+        return false;
+    }
+    return version->isQtQuickCompilerSupported(reason);
 }
 
-void BaseQtVersion::buildDebuggingHelper(ToolChain *tc, int tools)
+bool BaseQtVersion::isQtQuickCompilerSupported(QString *reason) const
 {
-    QTC_ASSERT(tc, return);
-    DebuggingHelperBuildTask *buildTask =
-            new DebuggingHelperBuildTask(this, tc, static_cast<DebuggingHelperBuildTask::Tools>(tools));
+    if (!isValid()) {
+        if (reason)
+            *reason = QCoreApplication::translate("BaseQtVersion", "Invalid Qt version.");
+        return false;
+    }
 
-    // pop up Application Output on error
-    buildTask->showOutputOnError(true);
+    if (qtVersion() < QtVersionNumber(5, 3, 0)) {
+        if (reason)
+            *reason = QCoreApplication::translate("BaseQtVersion", "Requires Qt 5.3.0 or newer.");
+        return false;
+    }
 
-    QFuture<void> task = QtConcurrent::run(&QtSupport::DebuggingHelperBuildTask::run, buildTask);
-    const QString taskName = QCoreApplication::translate("BaseQtVersion", "Building Debugging Helpers");
-    ProgressManager::addTask(task, taskName, "Qt::BuildHelpers");
+    const QString qtQuickCompilerExecutable =
+            HostOsInfo::withExecutableSuffix(binPath().toString() + QLatin1String("/qtquickcompiler"));
+    if (!QFileInfo::exists(qtQuickCompilerExecutable)) {
+        if (reason)
+            *reason = QCoreApplication::translate("BaseQtVersion", "This Qt Version does not contain Qt Quick Compiler.");
+        return false;
+    }
+
+    return true;
 }
 
-QList<FileName> BaseQtVersion::qtCorePaths(const QHash<QString,QString> &versionInfo, const QString &versionString)
+FileNameList BaseQtVersion::qtCorePaths(const QHash<QString,QString> &versionInfo, const QString &versionString)
 {
     QStringList dirs;
     dirs << qmakeProperty(versionInfo, "QT_INSTALL_LIBS")
          << qmakeProperty(versionInfo, "QT_INSTALL_BINS");
 
-    QList<FileName> staticLibs;
-    QList<FileName> dynamicLibs;
+    FileNameList staticLibs;
+    FileNameList dynamicLibs;
     foreach (const QString &dir, dirs) {
         if (dir.isEmpty())
             continue;
@@ -1489,7 +1735,7 @@ QList<FileName> BaseQtVersion::qtCorePaths(const QHash<QString,QString> &version
     return dynamicLibs;
 }
 
-QList<Abi> BaseQtVersion::qtAbisFromLibrary(const QList<FileName> &coreLibraries)
+QList<Abi> BaseQtVersion::qtAbisFromLibrary(const FileNameList &coreLibraries)
 {
     QList<Abi> res;
     foreach (const FileName &library, coreLibraries)
