@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -34,8 +29,10 @@
 #include "ui_fontsettingspage.h"
 
 #include <coreplugin/icore.h>
+#include <utils/fileutils.h>
 #include <utils/stringutils.h>
 #include <utils/qtcassert.h>
+#include <utils/theme/theme.h>
 
 #include <QFileDialog>
 #include <QFontDatabase>
@@ -110,7 +107,7 @@ private:
 class FontSettingsPagePrivate
 {
 public:
-    FontSettingsPagePrivate(const TextEditor::FormatDescriptions &fd,
+    FontSettingsPagePrivate(const FormatDescriptions &fd,
                             Core::Id id,
                             const QString &displayName,
                             const QString &category);
@@ -121,7 +118,7 @@ public:
     const QString m_displayName;
     const QString m_settingsGroup;
 
-    TextEditor::FormatDescriptions m_descriptions;
+    FormatDescriptions m_descriptions;
     FontSettings m_value;
     FontSettings m_lastValue;
     QPointer<QWidget> m_widget;
@@ -167,7 +164,7 @@ static QString createColorSchemeFileName(const QString &pattern)
 }
 
 // ------- FontSettingsPagePrivate
-FontSettingsPagePrivate::FontSettingsPagePrivate(const TextEditor::FormatDescriptions &fd,
+FontSettingsPagePrivate::FontSettingsPagePrivate(const FormatDescriptions &fd,
                                                  Core::Id id,
                                                  const QString &displayName,
                                                  const QString &category) :
@@ -179,38 +176,12 @@ FontSettingsPagePrivate::FontSettingsPagePrivate(const TextEditor::FormatDescrip
     m_schemeListModel(new SchemeListModel),
     m_refreshingSchemeList(false)
 {
-    bool settingsFound = false;
     QSettings *settings = Core::ICore::settings();
     if (settings)
-        settingsFound = m_value.fromSettings(m_settingsGroup, m_descriptions, settings);
+        m_value.fromSettings(m_settingsGroup, m_descriptions, settings);
 
-    if (!settingsFound) { // Apply defaults
-        foreach (const FormatDescription &f, m_descriptions) {
-            Format &format = m_value.formatFor(f.id());
-            format.setForeground(f.foreground());
-            format.setBackground(f.background());
-            format.setBold(f.format().bold());
-            format.setItalic(f.format().italic());
-        }
-    } else if (m_value.colorSchemeFileName().isEmpty()) {
-        // No color scheme was loaded, but one might be imported from the ini file
-        ColorScheme defaultScheme;
-        foreach (const FormatDescription &f, m_descriptions) {
-            Format &format = defaultScheme.formatFor(f.id());
-            format.setForeground(f.foreground());
-            format.setBackground(f.background());
-            format.setBold(f.format().bold());
-            format.setItalic(f.format().italic());
-        }
-        if (m_value.colorScheme() != defaultScheme) {
-            // Save it as a color scheme file
-            QString schemeFileName = createColorSchemeFileName(QLatin1String("customized%1.xml"));
-            if (!schemeFileName.isEmpty()) {
-                if (m_value.saveColorScheme(schemeFileName) && settings)
-                    m_value.toSettings(m_settingsGroup, settings);
-            }
-        }
-    }
+    if (m_value.colorSchemeFileName().isEmpty())
+        m_value.loadColorScheme(FontSettings::defaultSchemeFileName(), m_descriptions);
 
     m_lastValue = m_value;
 }
@@ -222,19 +193,57 @@ FontSettingsPagePrivate::~FontSettingsPagePrivate()
 
 
 // ------- FormatDescription
-FormatDescription::FormatDescription(TextStyle id, const QString &displayName, const QString &tooltipText, const QColor &foreground) :
-    m_id(id),
-    m_displayName(displayName),
-    m_tooltipText(tooltipText)
+FormatDescription::FormatDescription(TextStyle id,
+                                     const QString &displayName,
+                                     const QString &tooltipText,
+                                     const QColor &foreground,
+                                     FormatDescription::ShowControls showControls)
+    : m_id(id),
+      m_displayName(displayName),
+      m_tooltipText(tooltipText),
+      m_showControls(showControls)
 {
     m_format.setForeground(foreground);
 }
 
-FormatDescription::FormatDescription(TextStyle id, const QString &displayName, const QString &tooltipText, const Format &format) :
-    m_id(id),
-    m_format(format),
-    m_displayName(displayName),
-    m_tooltipText(tooltipText)
+FormatDescription::FormatDescription(TextStyle id,
+                                     const QString &displayName,
+                                     const QString &tooltipText,
+                                     const Format &format,
+                                     FormatDescription::ShowControls showControls)
+    : m_id(id),
+      m_format(format),
+      m_displayName(displayName),
+      m_tooltipText(tooltipText),
+      m_showControls(showControls)
+{
+}
+
+FormatDescription::FormatDescription(TextStyle id,
+                                     const QString &displayName,
+                                     const QString &tooltipText,
+                                     const QColor &underlineColor,
+                                     const QTextCharFormat::UnderlineStyle underlineStyle,
+                                     FormatDescription::ShowControls showControls)
+    : m_id(id),
+      m_displayName(displayName),
+      m_tooltipText(tooltipText),
+      m_showControls(showControls)
+{
+    m_format.setForeground(QColor());
+    m_format.setBackground(QColor());
+    m_format.setUnderlineColor(underlineColor);
+    m_format.setUnderlineStyle(underlineStyle);
+}
+
+FormatDescription::FormatDescription(TextStyle id,
+                                     const QString &displayName,
+                                     const QString &tooltipText,
+                                     FormatDescription::ShowControls showControls)
+    : m_id(id),
+      m_displayName(displayName),
+      m_tooltipText(tooltipText),
+      m_showControls(showControls)
 {
 }
 
@@ -252,8 +261,6 @@ QColor FormatDescription::foreground() const
             return QApplication::palette().foreground().color();
         else
             return m_format.foreground();
-    } else if (m_id == C_OCCURRENCES_UNUSED) {
-        return Qt::darkYellow;
     } else if (m_id == C_PARENTHESES) {
         return QColor(Qt::red);
     }
@@ -270,6 +277,8 @@ QColor FormatDescription::background() const
         return QColor(0xffef0b);
     } else if (m_id == C_PARENTHESES) {
         return QColor(0xb4, 0xee, 0xb4);
+    } else if (m_id == C_PARENTHESES_MISMATCH) {
+        return QColor(Qt::magenta);
     } else if (m_id == C_CURRENT_LINE || m_id == C_SEARCH_SCOPE) {
         const QPalette palette = QApplication::palette();
         const QColor &fg = palette.color(QPalette::Highlight);
@@ -311,6 +320,10 @@ QColor FormatDescription::background() const
     return QColor(); // invalid color
 }
 
+bool FormatDescription::showControl(FormatDescription::ShowControls showControl) const
+{
+    return m_showControls & showControl;
+}
 
 //  ------------ FontSettingsPage
 FontSettingsPage::FontSettingsPage(const FormatDescriptions &fd,
@@ -334,13 +347,12 @@ QWidget *FontSettingsPage::widget()
         d_ptr->m_widget = new QWidget;
         d_ptr->m_ui = new Ui::FontSettingsPage;
         d_ptr->m_ui->setupUi(d_ptr->m_widget);
+        d_ptr->m_ui->colorSchemeGroupBox->setTitle(
+                    tr("Color Scheme for Qt Creator Theme \"%1\"")
+                    .arg(Utils::creatorTheme()->displayName()));
         d_ptr->m_ui->schemeComboBox->setModel(d_ptr->m_schemeListModel);
 
-        QFontDatabase db;
-        const QStringList families = db.families();
-        d_ptr->m_ui->familyComboBox->addItems(families);
-        const int idx = families.indexOf(d_ptr->m_value.family());
-        d_ptr->m_ui->familyComboBox->setCurrentIndex(idx);
+        d_ptr->m_ui->fontComboBox->setCurrentFont(d_ptr->m_value.family());
 
         d_ptr->m_ui->antialias->setChecked(d_ptr->m_value.antialias());
         d_ptr->m_ui->zoomSpinBox->setValue(d_ptr->m_value.fontZoom());
@@ -349,13 +361,23 @@ QWidget *FontSettingsPage::widget()
         d_ptr->m_ui->schemeEdit->setBaseFont(d_ptr->m_value.font());
         d_ptr->m_ui->schemeEdit->setColorScheme(d_ptr->m_value.colorScheme());
 
-        connect(d_ptr->m_ui->familyComboBox, SIGNAL(currentIndexChanged(QString)), this, SLOT(fontFamilySelected(QString)));
-        connect(d_ptr->m_ui->sizeComboBox, SIGNAL(currentIndexChanged(QString)), this, SLOT(fontSizeSelected(QString)));
-        connect(d_ptr->m_ui->zoomSpinBox, SIGNAL(valueChanged(int)), this, SLOT(fontZoomChanged()));
-        connect(d_ptr->m_ui->antialias, SIGNAL(toggled(bool)), this, SLOT(antialiasChanged()));
-        connect(d_ptr->m_ui->schemeComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(colorSchemeSelected(int)));
-        connect(d_ptr->m_ui->copyButton, SIGNAL(clicked()), this, SLOT(copyColorScheme()));
-        connect(d_ptr->m_ui->deleteButton, SIGNAL(clicked()), this, SLOT(confirmDeleteColorScheme()));
+        connect(d_ptr->m_ui->fontComboBox, &QFontComboBox::currentFontChanged,
+                this, &FontSettingsPage::fontSelected);
+        connect(d_ptr->m_ui->sizeComboBox,
+                static_cast<void (QComboBox::*)(const QString &)>(&QComboBox::currentIndexChanged),
+                this, &FontSettingsPage::fontSizeSelected);
+        connect(d_ptr->m_ui->zoomSpinBox,
+                static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged),
+                this, &FontSettingsPage::fontZoomChanged);
+        connect(d_ptr->m_ui->antialias, &QCheckBox::toggled,
+                this, &FontSettingsPage::antialiasChanged);
+        connect(d_ptr->m_ui->schemeComboBox,
+                static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
+                this, &FontSettingsPage::colorSchemeSelected);
+        connect(d_ptr->m_ui->copyButton, &QPushButton::clicked,
+                this, &FontSettingsPage::openCopyColorSchemeDialog);
+        connect(d_ptr->m_ui->deleteButton, &QPushButton::clicked,
+                this, &FontSettingsPage::confirmDeleteColorScheme);
 
 
         updatePointSizes();
@@ -365,10 +387,10 @@ QWidget *FontSettingsPage::widget()
     return d_ptr->m_widget;
 }
 
-void FontSettingsPage::fontFamilySelected(const QString &family)
+void FontSettingsPage::fontSelected(const QFont &font)
 {
-    d_ptr->m_value.setFamily(family);
-    d_ptr->m_ui->schemeEdit->setBaseFont(d_ptr->m_value.font());
+    d_ptr->m_value.setFamily(font.family());
+    d_ptr->m_ui->schemeEdit->setBaseFont(font);
     updatePointSizes();
 }
 
@@ -392,7 +414,7 @@ void FontSettingsPage::updatePointSizes()
 QList<int> FontSettingsPage::pointSizesForSelectedFont() const
 {
     QFontDatabase db;
-    const QString familyName = d_ptr->m_ui->familyComboBox->currentText();
+    const QString familyName = d_ptr->m_ui->fontComboBox->currentFont().family();
     QList<int> sizeLst = db.pointSizes(familyName);
     if (!sizeLst.isEmpty())
         return sizeLst;
@@ -445,7 +467,7 @@ void FontSettingsPage::colorSchemeSelected(int index)
     d_ptr->m_ui->schemeEdit->setReadOnly(readOnly);
 }
 
-void FontSettingsPage::copyColorScheme()
+void FontSettingsPage::openCopyColorSchemeDialog()
 {
     QInputDialog *dialog = new QInputDialog(d_ptr->m_ui->copyButton->window());
     dialog->setAttribute(Qt::WA_DeleteOnClose);
@@ -454,7 +476,7 @@ void FontSettingsPage::copyColorScheme()
     dialog->setLabelText(tr("Color scheme name:"));
     dialog->setTextValue(tr("%1 (copy)").arg(d_ptr->m_value.colorScheme().displayName()));
 
-    connect(dialog, SIGNAL(textValueSelected(QString)), this, SLOT(copyColorScheme(QString)));
+    connect(dialog, &QInputDialog::textValueSelected, this, &FontSettingsPage::copyColorScheme);
     dialog->open();
 }
 
@@ -508,8 +530,8 @@ void FontSettingsPage::confirmDeleteColorScheme()
     messageBox->addButton(deleteButton, QMessageBox::AcceptRole);
     messageBox->setDefaultButton(deleteButton);
 
-    connect(deleteButton, SIGNAL(clicked()), messageBox, SLOT(accept()));
-    connect(messageBox, SIGNAL(accepted()), this, SLOT(deleteColorScheme()));
+    connect(deleteButton, &QAbstractButton::clicked, messageBox, &QDialog::accept);
+    connect(messageBox, &QDialog::accepted, this, &FontSettingsPage::deleteColorScheme);
     messageBox->setAttribute(Qt::WA_DeleteOnClose);
     messageBox->open();
 }
@@ -562,7 +584,7 @@ void FontSettingsPage::refreshColorSchemeList()
     int selected = 0;
 
     QStringList schemeList = styleDir.entryList();
-    QString defaultScheme = QFileInfo(FontSettings::defaultSchemeFileName()).fileName();
+    QString defaultScheme = Utils::FileName::fromString(FontSettings::defaultSchemeFileName()).fileName();
     if (schemeList.removeAll(defaultScheme))
         schemeList.prepend(defaultScheme);
     foreach (const QString &file, schemeList) {
@@ -623,7 +645,7 @@ void FontSettingsPage::saveSettings()
         d_ptr->m_lastValue = d_ptr->m_value;
         d_ptr->m_value.toSettings(d_ptr->m_settingsGroup, Core::ICore::settings());
 
-        QTimer::singleShot(0, this, SLOT(delayedChange()));
+        QTimer::singleShot(0, this, &FontSettingsPage::delayedChange);
     }
 }
 
