@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -1395,13 +1390,13 @@ void VcsBaseEditorWidget::setCommand(VcsCommand *command)
         d->m_progressIndicator->attachToWidget(this);
         connect(d->m_command.data(), &VcsCommand::finished,
                 this, &VcsBaseEditorWidget::hideProgressIndicator);
-        QTimer::singleShot(100, this, SLOT(showProgressIndicator()));
+        QTimer::singleShot(100, this, &VcsBaseEditorWidget::showProgressIndicator);
     }
 }
 
 void VcsBaseEditorWidget::setPlainText(const QString &text)
 {
-    TextEditorWidget::setPlainText(text);
+    textDocument()->setPlainText(text);
 }
 
 // Find the complete file from a diff relative specification.
@@ -1635,6 +1630,8 @@ void VcsBase::VcsBaseEditorWidget::testDiffFileResolving(const char *id)
     QFETCH(QByteArray, fileName);
     QTextDocument doc(QString::fromLatin1(header));
     QTextBlock block = doc.lastBlock();
+    // set source root for shadow builds
+    widget->setSource(QString::fromLatin1(SRC_DIR));
     QVERIFY(widget->fileNameFromDiffSpecification(block).endsWith(QString::fromLatin1(fileName)));
 
     delete editor;
