@@ -39,10 +39,14 @@
 #include "projectbuilddata.h"
 #include "rulesevaluationcontext.h"
 #include "transformer.h"
+
 #include <language/artifactproperties.h>
 #include <language/language.h>
 #include <language/loader.h>
+#include <language/propertymapinternal.h>
+#include <language/resolvedfilecontext.h>
 #include <logging/translator.h>
+#include <tools/fileinfo.h>
 #include <tools/persistence.h>
 #include <tools/propertyfinder.h>
 #include <tools/qbsassert.h>
@@ -213,7 +217,9 @@ void BuildGraphLoader::trackProjectChanges()
     // anew.
     if (hasBuildSystemFileChanged(buildSystemFiles, restoredProject->lastResolveTime)
             || hasEnvironmentChanged(restoredProject)
+            || hasCanonicalFilePathResultChanged(restoredProject)
             || hasFileExistsResultChanged(restoredProject)
+            || hasDirectoryEntriesResultChanged(restoredProject)
             || hasFileLastModifiedResultChanged(restoredProject)) {
         reResolvingNecessary = true;
     }
@@ -331,17 +337,34 @@ void BuildGraphLoader::trackProjectChanges()
 
 bool BuildGraphLoader::hasEnvironmentChanged(const TopLevelProjectConstPtr &restoredProject) const
 {
-    for (QHash<QString, QString>::ConstIterator it = restoredProject->usedEnvironment.constBegin();
-         it != restoredProject->usedEnvironment.constEnd(); ++it) {
-        const QString var = it.key();
-        const QString oldValue = it.value();
-        const QString newValue = m_environment.value(var);
-        if (newValue != oldValue) {
-            m_logger.qbsDebug() << QString::fromLatin1("Environment variable '%1' changed "
-                "from '%2' to '%3'. Must re-resolve project.").arg(var, oldValue, newValue);
+    QProcessEnvironment oldEnv = restoredProject->environment;
+    QProcessEnvironment newEnv = m_environment;
+
+    // HACK. Valgrind screws up our null-build benchmarker otherwise.
+    // TODO: Think about a (module-provided) whitelist.
+    oldEnv.remove(QLatin1String("LD_PRELOAD"));
+    newEnv.remove(QLatin1String("LD_PRELOAD"));
+
+    if (oldEnv != newEnv) {
+        m_logger.qbsDebug() << "Set of environment variables changed. Must re-resolve project.";
+        m_logger.qbsTrace() << "old: " << restoredProject->environment.toStringList() << "\nnew:"
+                            << m_environment.toStringList();
+        return true;
+    }
+    return false;
+}
+
+bool BuildGraphLoader::hasCanonicalFilePathResultChanged(const TopLevelProjectConstPtr &restoredProject) const
+{
+    for (auto it = restoredProject->canonicalFilePathResults.constBegin();
+         it != restoredProject->canonicalFilePathResults.constEnd(); ++it) {
+        if (QFileInfo(it.key()).canonicalFilePath() != it.value()) {
+            m_logger.qbsDebug() << "Canonical file path for file '" << it.key()
+                                << "' changed, must re-resolve project.";
             return true;
         }
     }
+
     return false;
 }
 
@@ -359,6 +382,23 @@ bool BuildGraphLoader::hasFileExistsResultChanged(const TopLevelProjectConstPtr
     return false;
 }
 
+bool BuildGraphLoader::hasDirectoryEntriesResultChanged(const TopLevelProjectConstPtr &restoredProject) const
+{
+    for (auto it = restoredProject->directoryEntriesResults.constBegin();
+         it != restoredProject->directoryEntriesResults.constEnd(); ++it) {
+        if (QDir(it.key().first).entryList(static_cast<QDir::Filters>(it.key().second), QDir::Name)
+                != it.value()) {
+            m_logger.qbsDebug() << "Entry list for directory '" << it.key().first
+                                << "' ("
+                                << static_cast<QDir::Filters>(it.key().second)
+                                << ") changed, must re-resolve project.";
+            return true;
+        }
+    }
+
+    return false;
+}
+
 bool BuildGraphLoader::hasFileLastModifiedResultChanged(const TopLevelProjectConstPtr &restoredProject) const
 {
     for (QHash<QString, FileTime>::ConstIterator it
@@ -428,8 +468,6 @@ void BuildGraphLoader::checkAllProductsForChanges(const QList<ResolvedProductPtr
         QList<ResolvedProductPtr> &changedProducts)
 {
     foreach (const ResolvedProductPtr &restoredProduct, restoredProducts) {
-        if (changedProducts.contains(restoredProduct))
-            continue;
         const ResolvedProductPtr newlyResolvedProduct
                 = newlyResolvedProductsByName.value(restoredProduct->uniqueName());
         if (!newlyResolvedProduct)
@@ -437,21 +475,25 @@ void BuildGraphLoader::checkAllProductsForChanges(const QList<ResolvedProductPtr
         if (newlyResolvedProduct->enabled != restoredProduct->enabled) {
             m_logger.qbsDebug() << "Condition of product '" << restoredProduct->uniqueName()
                                 << "' was changed, must set up build data from scratch";
-            changedProducts << restoredProduct;
+            if (!changedProducts.contains(restoredProduct))
+                changedProducts << restoredProduct;
             continue;
         }
 
-        if (!sourceArtifactSetsAreEqual(restoredProduct->allFiles(),
-                                        newlyResolvedProduct->allFiles())) {
-            m_logger.qbsDebug() << "File list of product '" << restoredProduct->uniqueName()
-                                << "' was changed.";
-            changedProducts += restoredProduct;
-            continue;
-        }
         if (checkProductForChanges(restoredProduct, newlyResolvedProduct)) {
             m_logger.qbsDebug() << "Product '" << restoredProduct->uniqueName()
                                 << "' was changed, must set up build data from scratch";
-            changedProducts << restoredProduct;
+            if (!changedProducts.contains(restoredProduct))
+                changedProducts << restoredProduct;
+            continue;
+        }
+
+        if (!sourceArtifactSetsAreEqual(restoredProduct->allEnabledFiles(),
+                                        newlyResolvedProduct->allEnabledFiles())) {
+            m_logger.qbsDebug() << "File list of product '" << restoredProduct->uniqueName()
+                                << "' was changed.";
+            if (!changedProducts.contains(restoredProduct))
+                changedProducts << restoredProduct;
         }
     }
 }
@@ -474,17 +516,29 @@ bool BuildGraphLoader::checkProductForChanges(const ResolvedProductPtr &restored
                                               const ResolvedProductPtr &newlyResolvedProduct)
 {
     // This check must come first, as it can prevent build data rescuing as a side effect.
-    // TODO: Similar special checks must be done for qbs.getEnv() and File.exists() in
+    // TODO: Similar special checks must be done for Environment.getEnv() and File.exists() in
     // commands (or possibly it could be reasonable to just forbid such "dynamic" constructs
     // within commands).
     if (checkForPropertyChanges(restoredProduct, newlyResolvedProduct))
         return true;
-
-    return !transformerListsAreEqual(restoredProduct->transformers,
-                                     newlyResolvedProduct->transformers)
-            || !ruleListsAreEqual(restoredProduct->rules.toList(),
-                                  newlyResolvedProduct->rules.toList())
-            || !dependenciesAreEqual(restoredProduct, newlyResolvedProduct);
+    if (!transformerListsAreEqual(restoredProduct->transformers,
+                                  newlyResolvedProduct->transformers)) {
+        return true;
+    }
+    if (!ruleListsAreEqual(restoredProduct->rules.toList(), newlyResolvedProduct->rules.toList()))
+        return true;
+    if (!dependenciesAreEqual(restoredProduct, newlyResolvedProduct))
+        return true;
+    const FileTime referenceTime = restoredProduct->topLevelProject()->lastResolveTime;
+    foreach (const ResolvedTransformerConstPtr &transformer, newlyResolvedProduct->transformers) {
+        if (!isPrepareScriptUpToDate(transformer->transform, referenceTime))
+            return true;
+    }
+    foreach (const RuleConstPtr &rule, newlyResolvedProduct->rules) {
+        if (!isPrepareScriptUpToDate(rule->prepareScript, referenceTime))
+            return true;
+    }
+    return false;
 }
 
 bool BuildGraphLoader::checkProductForInstallInfoChanges(const ResolvedProductPtr &restoredProduct,
@@ -715,6 +769,21 @@ bool BuildGraphLoader::isConfigCompatible()
     return true;
 }
 
+bool BuildGraphLoader::isPrepareScriptUpToDate(const ScriptFunctionConstPtr &script,
+                                               const FileTime &referenceTime)
+{
+    foreach (const JsImport &jsImport, script->fileContext->jsImports()) {
+        foreach (const QString &filePath, jsImport.filePaths) {
+            if (FileInfo(filePath).lastModified() > referenceTime) {
+                m_logger.qbsDebug() << "Change in import '" << filePath
+                        << "' potentially influences prepare script, marking as out of date";
+                return false;
+            }
+        }
+    }
+    return true;
+}
+
 void BuildGraphLoader::rescueOldBuildData(const ResolvedProductConstPtr &restoredProduct,
         const ResolvedProductPtr &newlyResolvedProduct, ProjectBuildData *oldBuildData,
         const ChildListHash &childLists, const AllRescuableArtifactData &existingRad)
