@@ -32,21 +32,29 @@
 
 #include "tst_language.h"
 
+#include "../../../../tests/auto/shared.h"
+
 #include <language/evaluator.h>
+#include <language/filecontext.h>
 #include <language/identifiersearch.h>
 #include <language/item.h>
 #include <language/itempool.h>
 #include <language/language.h>
+#include <language/propertymapinternal.h>
 #include <language/scriptengine.h>
+#include <language/value.h>
 #include <parser/qmljslexer_p.h>
 #include <parser/qmljsparser_p.h>
 #include <tools/scripttools.h>
 #include <tools/error.h>
+#include <tools/fileinfo.h>
 #include <tools/hostosinfo.h>
 #include <tools/profile.h>
 #include <tools/propertyfinder.h>
+#include <tools/settings.h>
 
 #include <QProcessEnvironment>
+#include <QVector>
 
 #include <algorithm>
 
@@ -67,7 +75,7 @@ TestLanguage::TestLanguage(ILogSink *logSink)
     qsrand(QTime::currentTime().msec());
     qRegisterMetaType<QList<bool> >("QList<bool>");
     defaultParameters.setBuildRoot("/some/build/directory");
-    defaultParameters.setPropertyCheckingMode(SetupProjectParameters::PropertyCheckingStrict);
+    defaultParameters.setPropertyCheckingMode(ErrorHandlingMode::Strict);
 }
 
 TestLanguage::~TestLanguage()
@@ -77,7 +85,7 @@ TestLanguage::~TestLanguage()
 QHash<QString, ResolvedProductPtr> TestLanguage::productsFromProject(ResolvedProjectPtr project)
 {
     QHash<QString, ResolvedProductPtr> result;
-    foreach (ResolvedProductPtr product, project->products)
+    foreach (const ResolvedProductPtr &product, project->allProducts())
         result.insert(product->name, product);
     return result;
 }
@@ -141,7 +149,7 @@ void TestLanguage::initTestCase()
     loader = new Loader(m_engine, m_logger);
     loader->setSearchPaths(QStringList()
                            << QLatin1String(SRCDIR "/../../../share/qbs"));
-    defaultParameters.setTopLevelProfile("qbs_autotests");
+    defaultParameters.setTopLevelProfile(profileName());
     defaultParameters.setBuildVariant("debug");
     defaultParameters.expandBuildConfiguration();
     QVERIFY(QFileInfo(m_wildcardsTestDirPath).isAbsolute());
@@ -349,6 +357,72 @@ void qbs::Internal::TestLanguage::dependencyOnAllProfiles()
     QCOMPARE(exceptionCaught, false);
 }
 
+void TestLanguage::derivedSubProject()
+{
+    bool exceptionCaught = false;
+    try {
+        SetupProjectParameters params = defaultParameters;
+        params.setProjectFilePath(testProject("derived-sub-project/project.qbs"));
+        const TopLevelProjectPtr project = loader->loadProject(params);
+        QVERIFY(project);
+        const QHash<QString, ResolvedProductPtr> products = productsFromProject(project);
+        QCOMPARE(products.count(), 1);
+    } catch (const ErrorInfo &e) {
+        exceptionCaught = true;
+        qDebug() << e.toString();
+    }
+    QCOMPARE(exceptionCaught, false);
+}
+
+void TestLanguage::defaultValue()
+{
+    bool exceptionCaught = false;
+    try {
+        SetupProjectParameters params = defaultParameters;
+        params.setProjectFilePath(testProject("defaultvalue/egon.qbs"));
+        QFETCH(QString, prop1Value);
+        QVariantMap overridden;
+        if (!prop1Value.isEmpty())
+            overridden.insert("lower.prop1", prop1Value);
+        params.setOverriddenValues(overridden);
+        TopLevelProjectPtr project = loader->loadProject(params);
+        QVERIFY(project);
+        QHash<QString, ResolvedProductPtr> products = productsFromProject(project);
+        QCOMPARE(products.count(), 2);
+        const ResolvedProductPtr product = products.value("egon");
+        QVERIFY(product);
+        QStringList propertyName = QStringList() << "modules" << "lower" << "prop2";
+        QVariant propertyValue = getConfigProperty(product->moduleProperties->value(), propertyName);
+        QFETCH(QVariant, expectedProp2Value);
+        QCOMPARE(propertyValue, expectedProp2Value);
+        propertyName = QStringList() << "modules" << "lower" << "listProp";
+        propertyValue = getConfigProperty(product->moduleProperties->value(), propertyName);
+        QFETCH(QVariant, expectedListPropValue);
+        QEXPECT_FAIL("controlling property not overwritten", "QBS-845", Continue);
+        QCOMPARE(propertyValue.toStringList(), expectedListPropValue.toStringList());
+    }
+    catch (const ErrorInfo &e) {
+        exceptionCaught = true;
+        qDebug() << e.toString();
+    }
+    QCOMPARE(exceptionCaught, false);
+}
+
+void TestLanguage::defaultValue_data()
+{
+    QTest::addColumn<QString>("prop1Value");
+    QTest::addColumn<QVariant>("expectedProp2Value");
+    QTest::addColumn<QVariant>("expectedListPropValue");
+    QTest::newRow("controlling property with random value") << "random" << QVariant("withoutBlubb")
+            << QVariant(QStringList({"other", "other"}));
+    QTest::newRow("controlling property with blubb value") << "blubb" << QVariant("withBlubb")
+            << QVariant(QStringList({"blubb", "other", "blubb", "other"}));
+    QTest::newRow("controlling property with egon value") << "egon" << QVariant("withEgon")
+            << QVariant(QStringList({"egon", "other"}));
+    QTest::newRow("controlling property not overwritten") << "" << QVariant("withBlubb")
+            << QVariant(QStringList({"blubb", "other", "blubb", "other"}));
+}
+
 void TestLanguage::environmentVariable()
 {
     bool exceptionCaught = false;
@@ -412,8 +486,14 @@ void TestLanguage::erroneousFiles_data()
             << "Array element at index 1 is undefined. String expected.";
     QTest::newRow("undeclared_item")
             << "Item 'cpp' is not declared.";
-    QTest::newRow("undeclared_property")
+    QTest::newRow("undeclared_property_wrapper")
             << "Property 'doesntexist' is not declared.";
+    QTest::newRow("undeclared_property_in_export_item")
+            << "Property 'blubb' is not declared.";
+    QTest::newRow("undeclared_property_in_export_item2")
+            << "Item 'something' is not declared.";
+    QTest::newRow("undeclared_property_in_export_item3")
+            << "Property 'blubb' is not declared.";
     QTest::newRow("unknown_item_type")
             << "Unexpected item type 'Narf'";
     QTest::newRow("invalid_child_item_type")
@@ -421,17 +501,25 @@ void TestLanguage::erroneousFiles_data()
     QTest::newRow("conflicting_fileTagsFilter")
             << "Conflicting fileTagsFilter in Group items";
     QTest::newRow("duplicate_sources")
-            << "Duplicate source file '.*main.cpp' "
-               "at .*duplicate_sources.qbs:4:12 and .*duplicate_sources.qbs:6:16.";
+            << "Duplicate source file '.*main.cpp'"
+               ".*duplicate_sources.qbs:4:12.*duplicate_sources.qbs:6:16.";
     QTest::newRow("duplicate_sources_wildcards")
-            << "Duplicate source file '.*duplicate_sources_wildcards.qbs' "
-               "at .*duplicate_sources_wildcards.qbs:4:12 "
-               "and .*duplicate_sources_wildcards.qbs:6:16.";
+            << "Duplicate source file '.*duplicate_sources_wildcards.qbs'"
+               ".*duplicate_sources_wildcards.qbs:4:12"
+               ".*duplicate_sources_wildcards.qbs:6:16.";
     QTest::newRow("oldQbsVersion")
             << "The project requires at least qbs version \\d+\\.\\d+.\\d+, "
                "but this is qbs version " QBS_VERSION ".";
     QTest::newRow("wrongQbsVersionFormat")
             << "The value '.*' of Project.minimumQbsVersion is not a valid version string.";
+    QTest::newRow("properties-item-with-invalid-condition")
+            << "TypeError: Result of expression 'cpp.nonexistingproperty'";
+    QTest::newRow("misused-inherited-property") << "Binding to non-item property";
+    QTest::newRow("undeclared_property_in_Properties_item") << "Item 'blubb' is not declared";
+    QTest::newRow("same-module-prefix1") << "The name of module 'prefix1' is equal to the first "
+                                            "component of the name of module 'prefix1.suffix'";
+    QTest::newRow("same-module-prefix2") << "The name of module 'prefix2' is equal to the first "
+                                            "component of the name of module 'prefix2.suffix'";
 }
 
 void TestLanguage::erroneousFiles()
@@ -449,7 +537,9 @@ void TestLanguage::erroneousFiles()
         }
         return;
     }
-    QFAIL("No error thrown on invalid input.");
+    QEXPECT_FAIL("misused-inherited-property", "QBS-847", Continue);
+    QEXPECT_FAIL("undeclared_property_in_Properties_item", "Too expensive to check", Continue);
+    QVERIFY(!"No error thrown on invalid input.");
 }
 
 void TestLanguage::exports()
@@ -460,13 +550,21 @@ void TestLanguage::exports()
         TopLevelProjectPtr project = loader->loadProject(defaultParameters);
         QVERIFY(project);
         QHash<QString, ResolvedProductPtr> products = productsFromProject(project);
-        QCOMPARE(products.count(), 9);
+        QCOMPARE(products.count(), 12);
         ResolvedProductPtr product;
         product = products.value("myapp");
         QVERIFY(product);
         QStringList propertyName = QStringList() << "modules" << "dummy" << "defines";
         QVariant propertyValue = getConfigProperty(product->moduleProperties->value(), propertyName);
-        QCOMPARE(propertyValue.toStringList(), QStringList() << "USE_MYLIB");
+        QCOMPARE(propertyValue.toStringList(), QStringList() << "BUILD_MYAPP" << "USE_MYLIB"
+                 << "USE_MYLIB2");
+        propertyName = QStringList() << "modules" << "dummy" << "includePaths";
+        QVariantList propertyValues = getConfigProperty(product->moduleProperties->value(),
+                                                        propertyName).toList();
+        QCOMPARE(propertyValues.count(), 3);
+        QVERIFY(propertyValues.at(0).toString().endsWith("/app"));
+        QVERIFY(propertyValues.at(1).toString().endsWith("/subdir/lib"));
+        QVERIFY(propertyValues.at(2).toString().endsWith("/subdir2/lib"));
 
         QCOMPARE(PropertyFinder().propertyValue(product->moduleProperties->value(), "dummy",
                                                 "productName").toString(), QString("myapp"));
@@ -477,6 +575,12 @@ void TestLanguage::exports()
         propertyValue = getConfigProperty(product->moduleProperties->value(), propertyName);
         QCOMPARE(propertyValue.toStringList(), QStringList() << "BUILD_MYLIB");
 
+        product = products.value("mylib2");
+        QVERIFY(product);
+        propertyName = QStringList() << "modules" << "dummy" << "defines";
+        propertyValue = getConfigProperty(product->moduleProperties->value(), propertyName);
+        QCOMPARE(propertyValue.toStringList(), QStringList() << "BUILD_MYLIB2");
+
         product = products.value("A");
         QVERIFY(product);
         QVERIFY(product->dependencies.contains(products.value("B")));
@@ -494,6 +598,11 @@ void TestLanguage::exports()
 
         product = products.value("myapp2");
         QVERIFY(product);
+        propertyName = QStringList() << "modules" << "dummy" << "cFlags";
+        propertyValue = getConfigProperty(product->moduleProperties->value(), propertyName);
+        QCOMPARE(propertyValue.toStringList(), QStringList()
+                 << "BASE_PRODUCTWITHINHERITEDEXPORTITEM"
+                 << "PRODUCT_PRODUCTWITHINHERITEDEXPORTITEM");
         propertyName = QStringList() << "modules" << "dummy" << "cxxFlags";
         propertyValue = getConfigProperty(product->moduleProperties->value(), propertyName);
         QCOMPARE(propertyValue.toStringList(), QStringList() << "-bar");
@@ -512,6 +621,12 @@ void TestLanguage::exports()
                                                 "productName").toString(), QString("myapp3"));
         QCOMPARE(PropertyFinder().propertyValue(product->moduleProperties->value(), "dummy",
                 "upperCaseProductName").toString(), QString("MYAPP3"));
+
+        // Verify we refer to the right "project" variable.
+        product = products.value("sub p2");
+        QVERIFY(product);
+        QCOMPARE(PropertyFinder().propertyValue(product->moduleProperties->value(), "dummy",
+                                                "someString").toString(), QString("sub1"));
     }
     catch (const ErrorInfo &e) {
         exceptionCaught = true;
@@ -697,7 +812,7 @@ void TestLanguage::identifierSearch_data()
                                   "Product {\n"
                                   "    name: {\n"
                                   "        var foo = 'bar';\n"
-                                  "        print(foo);\n"
+                                  "        console.info(foo);\n"
                                   "        return foo;\n"
                                   "    }\n"
                                   "}\n");
@@ -705,7 +820,7 @@ void TestLanguage::identifierSearch_data()
                                   "Product {\n"
                                   "    name: {\n"
                                   "        var foo = 'zort';\n"
-                                  "        print(narf + foo);\n"
+                                  "        console.info(narf + foo);\n"
                                   "        return foo;\n"
                                   "    }\n"
                                   "}\n");
@@ -713,7 +828,7 @@ void TestLanguage::identifierSearch_data()
                                   "Product {\n"
                                   "    name: {\n"
                                   "        var foo = 'narf';\n"
-                                  "        print(zort + foo);\n"
+                                  "        console.info(zort + foo);\n"
                                   "        return foo;\n"
                                   "    }\n"
                                   "}\n");
@@ -778,6 +893,25 @@ void TestLanguage::idUsage()
     QVERIFY(!exceptionCaught);
 }
 
+void TestLanguage::importCollection()
+{
+    bool exceptionCaught = false;
+    try {
+        defaultParameters.setProjectFilePath(testProject("import-collection/project.qbs"));
+        const TopLevelProjectPtr project = loader->loadProject(defaultParameters);
+        QVERIFY(project);
+        QHash<QString, ResolvedProductPtr> products = productsFromProject(project);
+        const ResolvedProductConstPtr product = products.value("da product");
+        QCOMPARE(product->productProperties.value("targetName").toString(),
+                 QLatin1String("C1f1C1f2C2f1C2f2"));
+    }
+    catch (const ErrorInfo &e) {
+        exceptionCaught = true;
+        qDebug() << e.toString();
+    }
+    QVERIFY(!exceptionCaught);
+}
+
 void TestLanguage::invalidBindingInDisabledItem()
 {
     bool exceptionCaught = false;
@@ -874,7 +1008,7 @@ void TestLanguage::jsExtensions()
     QScriptValue evaluated = m_engine->evaluate(code, file.fileName(), 1);
     if (m_engine->hasErrorOrException(evaluated)) {
         qDebug() << m_engine->uncaughtExceptionBacktrace();
-        QFAIL(qPrintable(evaluated.toString()));
+        QFAIL(qPrintable(m_engine->lastErrorString(evaluated)));
     }
 }
 
@@ -919,13 +1053,22 @@ void TestLanguage::moduleProperties_data()
     QTest::newRow("init") << QString() << QStringList();
     QTest::newRow("merge_lists")
             << "defines"
-            << (QStringList() << "THE_PRODUCT" << "QT_GUI" << "QT_CORE" << "QT_NETWORK");
+            << (QStringList() << "THE_PRODUCT" << "QT_CORE" << "QT_GUI" << "QT_NETWORK");
     QTest::newRow("merge_lists_and_values")
             << "defines"
-            << (QStringList() << "THE_PRODUCT" << "QT_NETWORK" << "QT_CORE" << "QT_GUI");
+            << (QStringList() << "THE_PRODUCT" << "QT_CORE" << "QT_GUI" << "QT_NETWORK");
     QTest::newRow("merge_lists_with_duplicates")
             << "cxxFlags"
             << (QStringList() << "-foo" << "BAR" << "-foo" << "BAZ");
+    QTest::newRow("merge_lists_with_prototype_values")
+            << "rpaths"
+            << (QStringList() << "/opt/qt/lib" << "$ORIGIN");
+    QTest::newRow("list_property_that_references_product")
+            << "listProp"
+            << (QStringList() << "x" << "123");
+    QTest::newRow("list_property_depending_on_overridden_property")
+            << "listProp2"
+            << (QStringList() << "PRODUCT_STUFF" << "DEFAULT_STUFF" << "EXTRA_STUFF");
     QTest::newRow("cleanup") << QString() << QStringList();
 }
 
@@ -943,6 +1086,7 @@ void TestLanguage::moduleProperties()
     QStringList valueStrings;
     foreach (const QVariant &v, values)
         valueStrings += v.toString();
+    QEXPECT_FAIL("list_property_depending_on_overridden_property", "QBS-845", Continue);
     QCOMPARE(valueStrings, expectedValues);
 }
 
@@ -1008,13 +1152,13 @@ void TestLanguage::modules_data()
                               << "dummyqt.network")
             << QString("guiProperty,networkProperty");
     QTest::newRow("deep_module_name")
-            << (QStringList() << "qbs" << "dummy" << "dummy.deep.moat")
+            << (QStringList() << "qbs" << "deepdummy.deep.moat" << "dummy")
             << QString("abysmal");
     QTest::newRow("deep_module_name_submodule_syntax1")
-            << (QStringList() << "qbs" << "dummy" << "dummy.deep.moat")
+            << (QStringList() << "qbs" << "deepdummy.deep.moat" << "dummy")
             << QString("abysmal");
     QTest::newRow("deep_module_name_submodule_syntax2")
-            << (QStringList() << "qbs" << "dummy" << "dummy.deep.moat")
+            << (QStringList() << "qbs" << "deepdummy.deep.moat" << "dummy")
             << QString("abysmal");
     QTest::newRow("dummy_twice")
             << (QStringList() << "qbs" << "dummy")
@@ -1041,6 +1185,57 @@ void TestLanguage::modules()
     QCOMPARE(product->productProperties.value("foo").toString(), expectedProductProperty);
 }
 
+void TestLanguage::nonRequiredProducts()
+{
+    bool exceptionCaught = false;
+    try {
+        SetupProjectParameters params = defaultParameters;
+        params.setProjectFilePath(testProject("non-required-products.qbs"));
+        QFETCH(bool, subProjectEnabled);
+        QFETCH(bool, dependeeEnabled);
+        QVariantMap overriddenValues;
+        if (!subProjectEnabled)
+            overriddenValues.insert("subproject.condition", false);
+        else if (!dependeeEnabled)
+            overriddenValues.insert("dependee.condition", false);
+        params.setOverriddenValues(overriddenValues);
+        const TopLevelProjectPtr project = loader->loadProject(params);
+        QVERIFY(project);
+        const auto products = productsFromProject(project);
+        QCOMPARE(products.count(), 4 + !!subProjectEnabled);
+        const ResolvedProductConstPtr dependee = products.value("dependee");
+        QCOMPARE(subProjectEnabled, !dependee.isNull());
+        if (dependee)
+            QCOMPARE(dependeeEnabled, dependee->enabled);
+        const ResolvedProductConstPtr depender = products.value("depender");
+        QVERIFY(depender);
+        const QStringList defines = PropertyFinder()
+                .propertyValue(depender->moduleProperties->value(), "dummy", "defines")
+                .toStringList();
+        QCOMPARE(subProjectEnabled && dependeeEnabled, defines.contains("WITH_DEPENDEE"));
+
+        for (const auto &name : QVector<const char *>({ "p3", "p2", "p1"})) {
+             const ResolvedProductConstPtr &product = products.value(name);
+             QVERIFY2(product, name);
+             QVERIFY2(!product->enabled, name);
+        }
+    }
+    catch (const ErrorInfo &e) {
+        exceptionCaught = true;
+        qDebug() << e.toString();
+    }
+    QCOMPARE(exceptionCaught, false);
+}
+
+void TestLanguage::nonRequiredProducts_data()
+{
+    QTest::addColumn<bool>("subProjectEnabled");
+    QTest::addColumn<bool>("dependeeEnabled");
+    QTest::newRow("dependee enabled") << true << true;
+    QTest::newRow("dependee disabled") << true << false;
+    QTest::newRow("sub project disabled") << false << true;
+}
+
 void TestLanguage::outerInGroup()
 {
     bool exceptionCaught = false;
@@ -1136,8 +1331,7 @@ void TestLanguage::profileValuesAndOverriddenValues()
         QCOMPARE(values.length(), 1);
         QCOMPARE(values.first().toString(), QString("IN_PROFILE"));
         values = pf.propertyValues(product->moduleProperties->value(), "dummy", "defines");
-        QCOMPARE(values.length(), 1);
-        QCOMPARE(values.first().toString(), QString("IN_FILE"));
+        QCOMPARE(values, QVariantList() << QLatin1String("IN_FILE") << QLatin1String("IN_PROFILE"));
         values = pf.propertyValues(product->moduleProperties->value(), "dummy", "cFlags");
         QCOMPARE(values.length(), 1);
         QCOMPARE(values.first().toString(), QString("OVERRIDDEN"));
@@ -1156,7 +1350,7 @@ void TestLanguage::productConditions()
         TopLevelProjectPtr project = loader->loadProject(defaultParameters);
         QVERIFY(project);
         QHash<QString, ResolvedProductPtr> products = productsFromProject(project);
-        QCOMPARE(products.count(), 4);
+        QCOMPARE(products.count(), 6);
         ResolvedProductPtr product;
         product = products.value("product_no_condition");
         QVERIFY(product);
@@ -1173,6 +1367,14 @@ void TestLanguage::productConditions()
         product = products.value("product_false_condition");
         QVERIFY(product);
         QVERIFY(!product->enabled);
+
+        product = products.value("product_probe_condition_false");
+        QVERIFY(product);
+        QVERIFY(!product->enabled);
+
+        product = products.value("product_probe_condition_true");
+        QVERIFY(product);
+        QVERIFY(product->enabled);
     }
     catch (const ErrorInfo &e) {
         exceptionCaught = true;
@@ -1283,6 +1485,12 @@ void TestLanguage::propertiesBlocks_data()
             << QString("dummy.defines")
             << (QStringList() << QString("OVERWRITTEN"))
             << QString();
+    QTest::newRow("using_derived_Properties_item") << "dummy.defines"
+            << (QStringList() << "string from MyProperties") << QString();
+    QTest::newRow("conditional-depends")
+            << QString("dummy.defines")
+            << QStringList()
+            << QString();
     QTest::newRow("cleanup") << QString() << QStringList() << QString();
 }
 
@@ -1306,6 +1514,149 @@ void TestLanguage::propertiesBlocks()
     }
 }
 
+void TestLanguage::propertiesBlockInGroup()
+{
+    bool exceptionCaught = false;
+    try {
+        defaultParameters.setProjectFilePath(testProject("properties-block-in-group.qbs"));
+        const TopLevelProjectPtr project = loader->loadProject(defaultParameters);
+        QVERIFY(project);
+        QCOMPARE(project->allProducts().count(), 1);
+        const ResolvedProductConstPtr product = project->allProducts().first();
+        const auto groupIt = std::find_if(product->groups.constBegin(), product->groups.constEnd(),
+                [](const GroupConstPtr &g) { return g->name == "the group"; });
+        QVERIFY(groupIt != product->groups.constEnd());
+        const QVariantMap propertyMap = (*groupIt)->properties->value();
+        const QVariantList value = PropertyFinder().propertyValues(propertyMap, "dummy", "defines");
+        QStringList stringListValue;
+        std::transform(value.constBegin(), value.constEnd(), std::back_inserter(stringListValue),
+                       [](const QVariant &v) { return v.toString(); });
+        QCOMPARE(stringListValue, QStringList() << "BASEDEF" << "FEATURE_ENABLED");
+    } catch (const ErrorInfo &e) {
+        exceptionCaught = true;
+        qDebug() << e.toString();
+    }
+    QCOMPARE(exceptionCaught, false);
+}
+
+void TestLanguage::qbsPropertiesInProjectCondition()
+{
+    bool exceptionCaught = false;
+    try {
+        defaultParameters.setProjectFilePath(
+                    testProject("qbs-properties-in-project-condition.qbs"));
+        const TopLevelProjectPtr project = loader->loadProject(defaultParameters);
+        QVERIFY(project);
+        const QHash<QString, ResolvedProductPtr> products = productsFromProject(project);
+        QCOMPARE(products.count(), 0);
+    } catch (const ErrorInfo &e) {
+        exceptionCaught = true;
+        qDebug() << e.toString();
+    }
+    QCOMPARE(exceptionCaught, false);
+}
+
+void TestLanguage::relaxedErrorMode()
+{
+    QFETCH(bool, strictMode);
+    try {
+        SetupProjectParameters params = defaultParameters;
+        params.setProjectFilePath(testProject("relaxed-error-mode/relaxed-error-mode.qbs"));
+        params.setProductErrorMode(strictMode ? ErrorHandlingMode::Strict
+                                              : ErrorHandlingMode::Relaxed);
+        const TopLevelProjectPtr project = loader->loadProject(params);
+        QVERIFY(!strictMode);
+        const auto productMap = productsFromProject(project);
+        const ResolvedProductConstPtr brokenProduct = productMap.value("broken");
+        QVERIFY(!brokenProduct->enabled);
+        QCOMPARE(brokenProduct->allFiles().count(), 0);
+        const ResolvedProductConstPtr dependerRequired = productMap.value("depender required");
+        QVERIFY(!dependerRequired->enabled);
+        QCOMPARE(dependerRequired->allFiles().count(), 1);
+        const ResolvedProductConstPtr dependerNonRequired
+                = productMap.value("depender nonrequired");
+        QVERIFY(dependerNonRequired->enabled);
+        QCOMPARE(dependerNonRequired->allFiles().count(), 1);
+        const ResolvedProductConstPtr recursiveDepender = productMap.value("recursive depender");
+        QVERIFY(!recursiveDepender->enabled);
+        QCOMPARE(recursiveDepender->allFiles().count(), 1);
+        const ResolvedProductConstPtr missingFile = productMap.value("missing file");
+        QVERIFY(missingFile->enabled);
+        QCOMPARE(missingFile->groups.count(), 1);
+        QVERIFY(!missingFile->groups.first()->enabled);
+        const ResolvedProductConstPtr fine = productMap.value("fine");
+        QVERIFY(fine->enabled);
+        QCOMPARE(fine->allFiles().count(), 1);
+    } catch (const ErrorInfo &e) {
+        QVERIFY2(strictMode, qPrintable(e.toString()));
+    }
+}
+
+void TestLanguage::relaxedErrorMode_data()
+{
+    QTest::addColumn<bool>("strictMode");
+
+    QTest::newRow("strict mode") << true;
+    QTest::newRow("relaxed mode") << false;
+}
+
+void TestLanguage::requiredAndNonRequiredDependencies()
+{
+    QFETCH(QString, projectFile);
+    QFETCH(bool, exceptionExpected);
+    try {
+        SetupProjectParameters params = defaultParameters;
+        const QString projectFilePath = "required-and-nonrequired-dependencies/" + projectFile;
+        params.setProjectFilePath(testProject(projectFilePath.toLocal8Bit()));
+        const TopLevelProjectConstPtr project = loader->loadProject(params);
+        QVERIFY(project);
+        QVERIFY(!exceptionExpected);
+    } catch (const ErrorInfo &e) {
+        QVERIFY(exceptionExpected);
+        QVERIFY2(e.toString().contains("validation error!"), qPrintable(e.toString()));
+    }
+}
+
+void TestLanguage::requiredAndNonRequiredDependencies_data()
+{
+    QTest::addColumn<QString>("projectFile");
+    QTest::addColumn<bool>("exceptionExpected");
+
+    QTest::newRow("same file") << "direct-dependencies.qbs" << true;
+    QTest::newRow("dependency via module") << "dependency-via-module.qbs" << true;
+    QTest::newRow("dependency via export") << "dependency-via-export.qbs" << true;
+    QTest::newRow("more indirection") << "complicated.qbs" << true;
+    QTest::newRow("required chain (module)") << "required-chain-module.qbs" << false;
+    QTest::newRow("required chain (export)") << "required-chain-export.qbs" << false;
+    QTest::newRow("required chain (export indirect)") << "required-chain-export-indirect.qbs"
+                                                      << false;
+}
+
+void TestLanguage::throwingProbe()
+{
+    QFETCH(bool, enableProbe);
+    try {
+        SetupProjectParameters params = defaultParameters;
+        params.setProjectFilePath(testProject("throwing-probe.qbs"));
+        QVariantMap properties;
+        properties.insert(QLatin1String("theProduct.enableProbe"), enableProbe);
+        params.setOverriddenValues(properties);
+        const TopLevelProjectPtr project = loader->loadProject(params);
+        QVERIFY(project);
+        QVERIFY(!enableProbe);
+    } catch (const ErrorInfo &e) {
+        QVERIFY2(enableProbe, qPrintable(e.toString()));
+    }
+}
+
+void TestLanguage::throwingProbe_data()
+{
+    QTest::addColumn<bool>("enableProbe");
+
+    QTest::newRow("enabled probe") << true;
+    QTest::newRow("disabled probe") << false;
+}
+
 void TestLanguage::qualifiedId()
 {
     QString str = "foo.bar.baz";
@@ -1328,6 +1679,33 @@ void TestLanguage::qualifiedId()
     QCOMPARE(ids, sorted);
 }
 
+void TestLanguage::recursiveProductDependencies()
+{
+    bool exceptionCaught = false;
+    try {
+        defaultParameters.setProjectFilePath(
+                    testProject("recursive-dependencies/recursive-dependencies.qbs"));
+        const TopLevelProjectPtr project = loader->loadProject(defaultParameters);
+        QVERIFY(project);
+        const QHash<QString, ResolvedProductPtr> products = productsFromProject(project);
+        QCOMPARE(products.count(), 4);
+        const ResolvedProductConstPtr p1 = products.value("p1");
+        QVERIFY(p1);
+        const ResolvedProductConstPtr p2 = products.value("p2");
+        QVERIFY(p2);
+        const ResolvedProductPtr p3 = products.value("p3");
+        QVERIFY(p3);
+        const ResolvedProductPtr p4 = products.value("p4");
+        QVERIFY(p4);
+        QVERIFY(p1->dependencies == QSet<ResolvedProductPtr>() << p3 << p4);
+        QVERIFY(p2->dependencies == QSet<ResolvedProductPtr>() << p3 << p4);
+    } catch (const ErrorInfo &e) {
+        exceptionCaught = true;
+        qDebug() << e.toString();
+    }
+    QCOMPARE(exceptionCaught, false);
+}
+
 void TestLanguage::fileTags_data()
 {
     QTest::addColumn<int>("numberOfGroups");
@@ -1481,6 +1859,14 @@ void TestLanguage::wildcards_data()
             << (QStringList() << "*.h")
             << QStringList()
             << (QStringList() << "subdir/foo.h" << "subdir/bar.h");
+    QTest::newRow(QByteArray("non-existing absolute path"))
+            << useGroup
+            << QStringList()
+            << QString()
+            << QString("/dir")
+            << (QStringList() << "*.whatever")
+            << QStringList()
+            << QStringList();
 }
 
 void TestLanguage::wildcards()
@@ -1563,7 +1949,7 @@ void TestLanguage::wildcards()
         QVERIFY(product);
         GroupPtr group;
         if (useGroup) {
-            QCOMPARE(product->groups.count(), 2);
+            QCOMPARE(product->groups.count(), HostOsInfo::isOsxHost() ? 4 : 3);
             foreach (const GroupPtr &rg, product->groups) {
                 if (rg->name == groupName) {
                     group = rg;
@@ -1571,7 +1957,7 @@ void TestLanguage::wildcards()
                 }
             }
         } else {
-            QCOMPARE(product->groups.count(), 1);
+            QCOMPARE(product->groups.count(), HostOsInfo::isOsxHost() ? 3 : 2);
             group = product->groups.first();
         }
         QVERIFY(group);
